
#-------------------------------------------------------------------------------
setMethod ('rederpost', 'RedPort',
           function (obj, method, ..., gdata=list(...)) { 
             aXML<-function(method, x){
               method <- paste(c("<methodName>",method,"</methodName>"), 
                               collapse = "",sep="")
               x <- lapply(x,function(arg){
                 paste(c("<string><![CDATA[",arg,"]]></string>"), collapse="", 
                       sep="")
               })
               x <- lapply(x,function(arg){
                 paste(c("<value>",arg,"</value>"), collapse="", sep="")
               })
               x <- lapply(x,function(arg){
                 paste(c("<param>",arg,"</param>"), collapse="", sep="")
               })
               x <- paste(unlist(x),collapse="", sep="")
               x <- paste("<params>",x,"</params>",collapse="", sep="")
               doc <- paste(c("<methodCall>",method,x,"</methodCall>"), 
                            collapse="", sep="")
               return(doc)
             }
             rdcall <- .simplePost(host=obj@host, port=obj@port,
                                   datatosend=aXML(method,gdata) )
             rdcall <- .postParser(rdcall)
             return(rdcall)
           }
)

#-------------------------------------------------------------------------------
setMethod ('ping', 'RedPort',
           function (obj) { 
             #Check if RedPort connection is available
             rval <- 0L
             connection <- try(suppressWarnings(
               socketConnection(host=obj@host, port=obj@port, blocking=TRUE)
             ), silent = TRUE)
             calltest1 <- !inherits(connection, "try-error")
             if(calltest1){
               calltest2 <- try(suppressWarnings(
                 .rederpost(obj,'RedHandler.ping')
                 ), silent=TRUE)
               if(is.numeric(calltest2) && length(calltest2)==1){
                 if(calltest2==1){
                   rval <- 1L
                 }
               }
               try(suppressWarnings(close(connection)), silent = TRUE)
               # try(suppressWarnings(closeAllConnections()), silent = TRUE)
             }
             return(rval)
           }
)

#-------------------------------------------------------------------------------
setMethod ('exitd', 'RedPort', 
           function (obj) { 
             if(ping(obj)==0)return(invisible())
             Sys.sleep(0.5)
             invisible(.rederpost(obj, 'RedHandler.exit'))
             Sys.sleep(0.5)
           }
)

#-------------------------------------------------------------------------------
setMethod ('resetd', 'RedPort', 
           function (obj) {
             if(ping(obj)==0)return(invisible())
             invisible(.rederpost(obj, 'RedHandler.reset'))
           }
)

#-------------------------------------------------------------------------------
setMethod ('version', 'RedPort', 
           function (obj) { 
             if(ping(obj)==0){
               return("RedeR/Java interface has not yet been initialized!")
             }
             return (.rederpost(obj, 'RedHandler.version'))
           }
)

#-------------------------------------------------------------------------------
setMethod ('calld', 'RedPort',
           
           function (obj, filepath='default', maxlag=20, checkcalls=FALSE) {
             
             #Check if the port is not in use by the app---------------------------
             if(ping(obj)==1){ 
               return("RedeR interface is already in use!")
             }
             if(!is.numeric(maxlag))maxlag=20
             
             #(1) get path to the 'reder.jar' file---------------------------------     
             if(filepath=="default"){
               filepath = system.file(package = "RedeR", "java/reder_v3.jar")
             }
             cmd="java -jar"
             
             #(2) check calld
             if(checkcalls){
               .checkJavaVersion()
               cat("Checking R-to-Java calls...\n")
               command = paste(cmd, shQuote(filepath), sep=' ')
               res <- system(command, ignore.stdout = FALSE, ignore.stderr = FALSE, wait=FALSE)
               if(is.numeric(res) && res[1]==0){
                 message("Call checks did not detect errors; otherwise please report \nany eventual error message to <mauro.a.castro at gmail.com>.\n")
               } else {
                 message(res)
                 message("Please report any eventual error message to <mauro.a.castro at gmail.com>")
               }
             } else {
               #(3)Execute 'calld' and update app settings in RedeR preferences:-----               
               command = paste(cmd, shQuote(filepath), "openshellDcall", obj@port, sep=' ')
               system(command, ignore.stdout = !checkcalls, ignore.stderr = !checkcalls, wait=FALSE)
             }

             #(4) Wait response from the app (implement a short-delay)-------------
             testInterface<-function(obj, maxlag=20){
               status="OFF"
               tdelta=0
               t0=proc.time()[3] #...used to start time delay! 
               pb <- txtProgressBar(style=2, char=".")
               while(status=="OFF"){
                 setTxtProgressBar(pb, tdelta/maxlag)
                 tdelta = proc.time()[3] - t0
                 if(tdelta>maxlag){
                   status="OFFON"
                 }
                 if(ping(obj)==1){
                   status="ON"
                   cat("\nRedeR is ready!\n")
                 }
               }      
               close(pb)
               #(4) ..send message if connection status is dubious!----------------------
               if(status=="OFFON") {
                 message("\nThe Java interface is not responding to initialization!")
                 message("Please, check whether Java is already installed in your machine.")
                 message("RedeR will need Java Runtime Environment (Java >=11)\n")
                 message("For a general diagnosis, re-run the 'calld' function with 'checkcalls=TRUE', for example: \n> calld(rdp, checkcalls=TRUE)")
               }
             }
             if(!checkcalls) testInterface(obj=obj,maxlag=maxlag)
 
           }
)
.checkJavaVersion <- function(){
  msg <- utils::packageVersion("RedeR")
  msg <- paste0("RedeR_",msg," will need Java Runtime Environment (Java >=11)\n")
  cat(msg)
  cat("Checking Java version installed on this system...\n")
  system2("java", args="-version")
}

#-------------------------------------------------------------------------------
setMethod ('updateGraph', 'RedPort', 
           function (obj) {
             if(ping(obj)==0)return(invisible())
             invisible (.rederpost(obj, 'RedHandler.updateGraph'))
           }
)

#-------------------------------------------------------------------------------
setMethod ('updateCoordXY', 'RedPort', 
           function (obj, g, delNodes=FALSE, delEdges=FALSE){
             if(ping(obj)==0) stop("No RedeR interface is availble.")
             if(!igraph::is_igraph(g))
               stop("'g' object should be an igraph object!")
             gcls <- class(g)
             invisible(.rederpost(obj,'RedHandler.stopRelax'))
             invisible(.rederpost(obj,'RedHandler.lockDragAndZoom'))
             gg <- getGraph(obj, attribs="plain")
             invisible(.rederpost(obj,'RedHandler.unLockDragAndZoom'))
             if(vcount(gg)==0){
               tp1 <- "No vertex appears to be available in the ReadR interface.\n"
               tp2 <- "The 'updateCoordXY' function aims to update node coordinates "
               tp3 <- "in the 'g' object using graph coordinates from the RedeR interface!"
               stop(tp1,tp2,tp3)
             }
             if(is.null(V(g)$name)){
               if(vcount(g)!=vcount(gg)){
                 tp1 <- "Number of vertices in 'g' and in the RedeR interface is different,\n"
                 tp2 <- "and vertices in the 'g' object are not named.\n"
                 tp3 <- "Not possible to match these graphs."
                 stop(tp1,tp2,tp3)
               }
               V(g)$name <- as.character(1:vcount(g))
               V(gg)$name  <- as.character(1:vcount(gg))
               tp1 <- "Note: vertices in the 'g' object are not named.\n"
               tp2 <- "It will be used the vertices numbering as IDs."
               warning(tp1,tp2)
             }
             if(sum(V(g)$name%in%V(gg)$name)==0){
               stop("No vertices in 'g' seem to match vertices in the ReadeR interface.")
             }
             #--- intersect vertices
             nodes <- intersect(V(g)$name,V(gg)$name)
             idx <- which(!V(g)$name%in%nodes)
             if(delNodes && length(idx)>0){
               message("Deleting ",length(idx), " vertices from 'g'!")
               g <- delete_vertices(g, v=idx)
             }
             idx <- which(!V(gg)$name%in%nodes)
             if(length(idx)>0){
               gg <- delete_vertices(gg, v=idx)
             }
             #--- update coords
             message("Updating 'coordX' and 'coordY' of 'g' vertices!")
             if(is.null(V(g)$coordX)) V(g)$coordX <- 0
             if(is.null(V(g)$coordY)) V(g)$coordY <- 0
             idx <- match(V(gg)$name, V(g)$name)
             V(g)$coordX[idx] <- V(gg)$coordX
             V(g)$coordY[idx] <- V(gg)$coordY
             #--- delete edges
             if(delEdges){
               eg <- .get_elist(g)
               egg <- .get_elist(gg)
               if(nrow(eg)>0 && nrow(egg)>0){
                 idx <- eg$ID12%in%egg$ID12|eg$ID12%in%egg$ID21
                 idx <- which(!idx)
                 if(length(idx)>0){
                   message("Deleting ",length(idx), " edges from 'g'!")
                   g <- delete_edges(g, idx)
                 }
               }               
             }
             #--- update zoom and class
             g$gzoom=.rederpost(obj,'RedHandler.getZoom')
             class(g) <- gcls
             return(g)
           }
)
.get_elist <- function(g){
  el <- as_edgelist(g)
  if(nrow(el)>0){
    el <- data.frame(as_edgelist(g), stringsAsFactors = FALSE)
    colnames(el) <- c("Node1","Node2")
    el$ID12 <- paste0(el$Node1,"|",el$Node2)
    el$ID21 <- paste0(el$Node2,"|",el$Node1) 
  }
  return(el)
}

#-------------------------------------------------------------------------------
setMethod ('getGraph', 'RedPort', 
           function (obj, status="all", type="node", attribs="plain") {
             if(ping(obj)==0)return(igraph::make_empty_graph(n=0, directed=FALSE))
             
             #check loaded igraph
             igraph.check()
             
             #Get graph objects from RedeR app
             nodes = getNodes(obj, status, type)
             edges = getEdges(obj, status, type) 
             
             #Build igraph object
             if(length(nodes)==1 && nodes==""){
               g = igraph::make_empty_graph(n=0, directed=FALSE)
               return(g) 
             } else if(length(edges)==1 && edges ==""){
               edges = NULL
               g	   = igraph::make_empty_graph(n=length(nodes), directed=FALSE)
               g     = igraph::set_vertex_attr(g, "name", value=nodes)
             } else {
               nodes	= data.frame(name=nodes, stringsAsFactors=FALSE)
               edges 	= matrix(edges,ncol=2, byrow=TRUE)
               colnames(edges) = c("NodeA","NodeB")
               edges 	= data.frame(edges,stringsAsFactors=FALSE)
               g  	 	= igraph::graph_from_data_frame(edges, directed=FALSE, vertices=nodes)
             }  
             
             #Add required attributes    
             if(attribs=="minimal"){
               return(g) 
             } else if(attribs=="plain"){
               nodeX = .getNodeX(obj, status, type )
               nodeY = .getNodeY(obj, status, type )
               g = igraph::set_vertex_attr(g, "coordX", value=nodeX)
               g = igraph::set_vertex_attr(g, "coordY", value=nodeY)
               g$zoom = .rederpost(obj,'RedHandler.getZoom')
               return(g)
             } else if(attribs=="all"){
               nodeAlias = .getNodeAliases(obj, status, type ) 
               nodeBend  = .getNodeBend(obj, status, type )     
               nodeX     = .getNodeX(obj, status, type )
               nodeY     = .getNodeY(obj, status, type )
               nodeSize  = .getNodeSize(obj, status, type )
               nodeShape = .getNodeShape(obj, status, type )
               nodeColor = .getNodeColor(obj, status, type )
               nodeWeight     = .getNodeWeight(obj, status, type )
               nodeLineWidth  = .getNodeLineWidth(obj, status, type )
               nodeLineColor  = .getNodeLineColor(obj, status, type ) 
               nodeFontSize   = .getNodeFontSize(obj, status, type )
               nodeFontColor  = .getNodeFontColor(obj, status, type )
               g     = igraph::set_vertex_attr(g, "nodeAlias",  value=nodeAlias)
               g     = igraph::set_vertex_attr(g, "nodeBend",   value=nodeBend)
               g     = igraph::set_vertex_attr(g, "coordX",   value=nodeX)
               g     = igraph::set_vertex_attr(g, "coordY",   value=nodeY)
               g     = igraph::set_vertex_attr(g, "nodeSize",   value=nodeSize)   
               g     = igraph::set_vertex_attr(g, "nodeShape",  value=nodeShape)
               g     = igraph::set_vertex_attr(g, "nodeColor",  value=nodeColor)
               g     = igraph::set_vertex_attr(g, "nodeWeight", value=nodeWeight)
               g     = igraph::set_vertex_attr(g, "nodeLineWidth",  value=nodeLineWidth)
               g     = igraph::set_vertex_attr(g, "nodeLineColor",  value=nodeLineColor)
               g     = igraph::set_vertex_attr(g, "nodeFontSize",   value=nodeFontSize)
               g     = igraph::set_vertex_attr(g, "nodeFontColor",  value=nodeFontColor)
               g$zoom <- .rederpost(obj,'RedHandler.getZoom')
               #..get edge attrs. if present!
               if(!is.null(edges)){
                 arrowDirection = .getArrowDirection (obj, status, type )  
                 edgeWeight     = .getEdgeWeight(obj, status, type )
                 edgeWidth      = .getEdgeWidth(obj, status, type )
                 edgeColor      = .getEdgeColor(obj, status, type )
                 edgeType       = .getEdgeType(obj, status, type )
                 g     = igraph::set_edge_attr(g, "arrowDirection", value=arrowDirection)
                 #g     = igraph::set_edge_attr(g, "arrowLength",    value= arrowLength)
                 #g     = igraph::set_edge_attr(g, "arrowAngle",     value= arrowAngle)
                 g     = igraph::set_edge_attr(g, "edgeWeight",     value=edgeWeight)
                 g     = igraph::set_edge_attr(g, "edgeWidth",      value=edgeWidth)
                 g     = igraph::set_edge_attr(g, "edgeColor",      value=edgeColor)
                 g     = igraph::set_edge_attr(g, "edgeType",       value=edgeType)
               }          
               return(g)       
             }
           }
)

#-------------------------------------------------------------------------------
setMethod ('addGraph', 'RedPort', 
           function (obj, g, layout, gscale=75, gcoord=c(50,50), 
                     zoom=NULL, gzoom=NULL, 
                     isNested=FALSE, isAnchored=TRUE, isAssigned=FALSE, 
                     isNest=NULL, isAnchor=NULL, isAssign=NULL, 
                     loadEdges=TRUE, ntransform=FALSE, 
                     parent=NULL, theme='tm0', .callchecks=TRUE) {
             
             #Callcheck
             if(.callchecks) if(ping(obj)==0)return(invisible())
             
             #check loaded igraph
             igraph.check()
             
             #deprecated parameters
             if(!is.null(isNest)){
               warning("'isNest' attribute is deprecated; please use 'isNested'.")
               isNested <- isNest
             }
             if(!is.null(isAnchor)){
               warning("'isAnchor' attribute is deprecated; please use 'isAnchored'.")
               isAnchored <- isAnchor
             }
             if(!is.null(isAssign)){
               warning("'isAssign' attribute is deprecated; please use 'isAssigned'.")
               isAssigned <- isAssign
             }   
             #Check igraph object-----------------------------------------------
             if(!igraph::is_igraph(g))
               stop("'g' should be an igraph object!")
             
             #Check igraph direction
             if(igraph::is_directed(g)) g <- check.igraph.direction(g)
             
             #Check igraph format
             g <- check.igraph.format(g)

             #Check igraph size-------------------------------------------------
             #...if empty graph!
             if(igraph::vcount(g)==0){
               g=igraph::add_vertices(g, 2)
               if((!is.null(G(g,"isNested")) && G(g,"isNested"))||isNested){
                 V(g)$name=c("NN0<$$>","NN1<$$>")
               } else {
                 V(g)$name=c("MM0<$$>","MM1<$$>")
               }
               layout=igraph::layout.random(g)
             }
             
             #Get names from vertex if not available------------------------------
             if(is.null(V(g)$name)){
               V(g)$name=as.character(V(g))
             } else {
               V(g)$name=as.character(V(g)$name)
             }
             
             #Add empty node if igraph::vcount(g)==1 
             #..to keep data type as vector during server connection!   
             if(igraph::vcount(g)==1){
               g=igraph::add_vertices(g, 1)
               if((!is.null(G(g,"isNested")) && G(g,"isNested"))||isNested){
                 V(g)$name[2]="NN0<$$>"   		
               } else {
                 V(g)$name[2]="MM0<$$>"  		
               }
               layout=igraph::layout.random(g)
               if(is.character(V(g)$nodeAlias))V(g)$nodeAlias=V(g)$nodeAlias[1]
               if(is.numeric(V(g)$coordX))V(g)$coordX=V(g)$coordX[1]
               if(is.numeric(V(g)$coordY))V(g)$coordY=V(g)$coordY[1]
               if(is.numeric(V(g)$nodeBend))V(g)$nodeBend=V(g)$nodeBend[1]   
               if(is.numeric(V(g)$nodeSize))V(g)$nodeSize=V(g)$nodeSize[1]
               if(is.character(V(g)$nodeShape))V(g)$nodeShape=V(g)$nodeShape[1]
               if(is.character(V(g)$nodeColor))V(g)$nodeColor=V(g)$nodeColor[1]
               if(is.numeric(V(g)$nodeWeight))V(g)$nodeWeight=V(g)$nodeWeight[1]
               if(is.numeric(V(g)$nodeLineWidth))V(g)$nodeLineWidth=V(g)$nodeLineWidth[1]
               if(is.character(V(g)$nodeLineColor))V(g)$nodeLineColor=V(g)$nodeLineColor[1]
               if(is.numeric(V(g)$nodeFontSize))V(g)$nodeFontSize=V(g)$nodeFontSize[1]
               if(is.character(V(g)$nodeFontColor))V(g)$nodeFontColor=V(g)$nodeFontColor[1]
             }
             
             #Check logical args.
             isNested=ifelse(!is.logical(isNested),FALSE,isNested)
             isAnchored=ifelse(!is.logical(isAnchored),TRUE,isAnchored)
             isAssigned=ifelse(!is.logical(isAssigned),FALSE,isAssigned)
             loadEdges=ifelse(!is.logical(loadEdges),TRUE,loadEdges)
             
             #Check/get graph attribs.
             if(is.numeric(G(g,"gscale")) )gscale=G(g,"gscale")
             if(is.logical(G(g,"isNested")) )isNested=G(g,"isNested")
             if(is.logical(G(g,"isAnchored")) )isAnchored=G(g,"isAnchored")
             if(is.logical(G(g,"isAssigned")) )isAssigned=G(g,"isAssigned")
             if(is.logical(G(g,"loadEdges")))loadEdges=G(g,"loadEdges")  
             if(is.numeric(G(g,"zoom")) ) zoom=G(g,"zoom")
             if(is.numeric(G(g,"gzoom")) ) gzoom=G(g,"gzoom")
             #---gzoom overrides zoom and has effect only on objects
             if(!is.null(gzoom))zoom <- gzoom
             
             #--- set bg
             if(is.character(G(g,"bgColor"))){
               bgColor=G(g,"bgColor")
             } else {
               bgColor=NULL
             } 
             
             #..nested assigments are not straightforward for R-J!!
             if(isAssigned && isNested){
               temp=V(g)$name
               if(is.null(V(g)$nodeAlias)){
                 V(g)$nodeAlias=temp
               } else {
                 idx=is.na(V(g)$nodeAlias)
                 V(g)$nodeAlias[idx]=temp[idx]
               }
               id="N001"
               if(is.character(parent))id=parent
               V(g)$name=paste(temp,".$",id,sep="")
             }
             # if theme is list, gatt gets all attributes for further settings
             # to address to nestNodes function, and also set some local graph attributes
             if(is.list(theme)){
               gatt=theme
               if(is.null(theme$theme)){
                 theme=0
               } else {
                 theme=theme$theme
               }
               if(is.numeric(gatt$zoom))zoom=gatt$zoom[1]
               if(is.numeric(gatt$gscale))gscale=gatt$gscale[1]
               if(is.numeric(gatt$gcoord))gcoord=gatt$gcoord
               if(is.logical(gatt$isNested))isNested=gatt$isNested[1]
               if(is.logical(gatt$isAnchored))isAnchored=gatt$isAnchored[1]
               if(is.logical(gatt$isAssigned))isAssigned=gatt$isAssigned[1]
               if(is.logical(gatt$loadEdges))loadEdges=gatt$loadEdges[1]
               if(is.character(gatt$parent))parent=gatt$parent[1]
             } else {
               gatt<-list()   
             }
             # but if g has nest attributes, it is prioritized over theme!
             nestColor=NULL
             if(is.character(G(g,"nestColor")) )gatt$nestColor=G(g,"nestColor") 
             nestAlias=NULL
             if(is.character(G(g,"nestAlias")))gatt$nestAlias=G(g,"nestAlias")
             nestFontSize=NULL
             if(is.numeric(G(g,"nestFontSize")))gatt$nestFontSize=G(g,"nestFontSize") 
             nestFontColor=NULL 
             if(is.character(G(g,"nestFontColor")))gatt$nestFontColor=G(g,"nestFontColor")    
             nestFontX=NULL 
             if(is.numeric(G(g,"nestFontX")))gatt$nestFontX=G(g,"nestFontX")
             nestFontY=NULL 
             if(is.numeric(G(g,"nestFontY")))gatt$nestFontY=G(g,"nestFontY")
             nestShape=NULL 
             if(is.character(G(g,"nestShape")))gatt$nestShape=G(g,"nestShape")
             nestSize=NULL 
             if(is.numeric(G(g,"nestSize"))) gatt$nestSize=G(g,"nestSize")  
             nestLineWidth=NULL 
             if(is.numeric(G(g,"nestLineWidth")))gatt$nestLineWidth=G(g,"nestLineWidth")  
             nestLineColor =NULL 
             if(is.character(G(g,"nestLineColor")))gatt$nestLineColor=G(g,"nestLineColor")
             nestLineType =NULL 
             if(is.character(G(g,"nestLineType")))gatt$nestLineType=G(g,"nestLineType")
             nestImage='plain'
             if(is.character(G(g,"nestImage")))nestImage=G(g,"nestImage")
             
             #Check gcoord option-----------------------------------------------
             c1=!is.numeric(gcoord)
             c2=!length(gcoord)==2
             if(c1 || c2){
               if(G(g,"gcoord")!=NULL){
                 warning("NOTE: attribute 'gcoord' is not set properly in the igraph object!")
               }    	
               stop("gcoord must be a numeric vector of length=2.")
             }
             
             #Set zoom if available
             if(!is.null(zoom)){
               zoom=zoom[1]
               if(!is.numeric(zoom)){
                 warning("NOTE: graph 'zoom' must be provided as numerics (range: 0.0 to 100.0)!")
               } else if(is.na(zoom)){
                 warning("NOTE: invalid graph 'zoom': 'NA' found'!")
               } else {            
                 message("*** Setting graph 'zoom'...") 
                 invisible( .rederexpresspost(obj, 'RedHandler.setZoom', zoom) )
               }
             } else {
               zoom=100
             }
             
             #PS. the following methods must be used only in low-level calls!
             message('*** Uploading graph to RedeR server...')
             
             #Check layout option----------------------------------------------
             layoutOldNodes=TRUE
             flagCoordXY=TRUE
             if(missing(layout)){
               b1 <- !is.null(V(g)$coordX) && !is.null(V(g)$coordY)
               b2 <- length(V(g)$coordX)==length(V(g)$coordY)
               if(b1 && b2){
                 layout <- cbind(V(g)$coordX,V(g)$coordY)
               } else {
                 layout <- igraph::layout_randomly(g)
                 layoutOldNodes=FALSE
                 flagCoordXY=FALSE
               }
             } else if(is.null(layout)){
               vattrbs <- igraph::vertex_attr_names(g)
               if("coordX"%in%vattrbs) g <- igraph::delete_vertex_attr(g,"coordX")
               if("coordY"%in%vattrbs) g <- igraph::delete_vertex_attr(g,"coordY")
               flagCoordXY=FALSE
             }
             if(!is.null(layout)){
               if(!is.matrix(layout)){
                 stop("Layout must be provided as matrix!")
               } else if(ncol(layout)!=2){
                 stop("Layout matrix must have 2 cols (i.e. x and y coords)!")
               } else if(nrow(layout)!=igraph::vcount(g) ) {
                 stop("Layout does not match graph vertices: inconsistent row number!")
               } else {
                 s1 <- !is.numeric(gscale)
                 s2 <- is.null(gscale)
                 s3 <- is.na(gscale)
                 if(s1 || s2 || s3){
                   warning("NOTE: attribute 'gscale' is not set properly; must be <numeric> of length=1!")
                   gscale = 75
                 }
                 if(!is.null(zoom) && is.null(gzoom)){
                   gscale <- gscale*(zoom[1]/100)
                 }
                 pScale <- .rederpost(obj,'RedHandler.getPanelScale')
                 pScale <- as.numeric(pScale)
                 if(is.numeric(pScale)){
                   pScale <- pScale[1]*(gscale[1]/100)
                 } else {
                   pScale <- 500
                 }
                 if(isNested)pScale=pScale/sqrt(2)
                 if(anyNA(layout)){
                   idx <- is.na(layout)
                   rg <- range(layout, na.rm = T)
                   layout[idx] <- runif(sum(idx),rg[1],rg[2])
                 }
                 l1 <- diff(range(layout[,1]))
                 l2 <- diff(range(layout[,2]))
                 if(l1>l2){
                   l2 <- l2/l1*pScale
                   l1 <- pScale
                 } else {
                   l1 <- l1/l2*pScale
                   l2 <- pScale
                   }
                 layout <- igraph::norm_coords(layout, xmin=0, xmax=l1, ymin=0, ymax=l2)
                 V(g)$coordX <- layout[,1]
                 V(g)$coordY <- layout[,2]
               }       
             }
             
             #Set/get nodes and edges to submit to the app------------------------        
             nodes = V(g)$name
             edges = igraph::as_edgelist(g, names=TRUE)
             edges = cbind(as.character(edges[,1]),as.character(edges[,2]))    
             
             #Set graph background color if available
             if(!is.null(bgColor)){ 
               bgColor=bgColor[1]
               if(!is.color(bgColor)){
                 warning("NOTE: graph 'color' must be provided as hexadecimal.")
               } else if(is.na(bgColor)){
                 warning("NOTE: invalid graph 'color': 'NA' found'!")
               } else {
                 bgColor <- colorRampPalette(bgColor, alpha=TRUE)(length(bgColor))
                 message("** ... graph background 'color'")
                 invisible( .rederexpresspost(obj, 'RedHandler.setBackground', bgColor) )
               }      
             }  
             
             #Add nodes, edges and set attributes (if available)------------------------
             if(igraph::vcount(g)>0)message("** ... nodes!") 
             if(igraph::ecount(g)>0)message("** ... edges!")
             if( length(igraph::vertex_attr_names(g))>3 || flagCoordXY){
               message('*** Uploading node attributes...')   
             }
             nodeAlias      = V(g)$nodeAlias 
             coordX         = V(g)$coordX
             coordY         = V(g)$coordY 
             nodeBend       = V(g)$nodeBend       
             nodeSize       = V(g)$nodeSize
             nodeShape      = V(g)$nodeShape
             nodeColor      = V(g)$nodeColor
             nodeWeight     = V(g)$nodeWeight
             nodeLineWidth  = V(g)$nodeLineWidth
             nodeLineColor  = V(g)$nodeLineColor
             nodeFontSize   = V(g)$nodeFontSize
             nodeFontColor  = V(g)$nodeFontColor        
             #nodeAlias
             if(!is.null(nodeAlias) && length(nodeAlias)>0){       
               c1=!is.character(nodeAlias)
               if(c1){
                 warning("NOTE: node 'alias' must be provided as character!")
                 nodeAlias=as.character(c('',''))
               } else {
                 message("** ... node 'alias'")
                 nodeAlias[is.na(nodeAlias)]=nodes[is.na(nodeAlias)]
               }  
             } else {	
               al=c(V(g)$name[1],V(g)$name[2])
               nodeAlias=as.character(al)
             }      
             #Node coords.
             c1=length(coordX)>0 && length(coordY)>0
             c2=!is.null(coordX) && !is.null(coordY)
             if(c1 && c2){
               c1=!is.numeric(coordX)
               c2=!is.numeric(coordY)
               if(c1 && c2){
                 warning("NOTE: node coords must be provided as numerics!")
                 coordX=as.numeric(c(10,10))
                 coordY=as.numeric(c(10,10,10))
               } else if(sum(is.na(coordX))>0 || sum(is.na(coordY))>0 ){
                 warning("NOTE: invalid node 'coordX' or 'coordY': 'NA' found'!")
                 coordX=as.numeric(c(10,10))
                 coordY=as.numeric(c(10,10,10))
               } else {
                 if(flagCoordXY) message("** ... node 'coords'") 
                 coordX=as.numeric(coordX) 
                 coordY=as.numeric(coordY) 
               }      
             } else {
               coordX=as.numeric(c(10,10))
               coordY=as.numeric(c(10,10,10))      
             }
             #nodeBend
             if(!is.null(nodeBend) && length(nodeBend)>0){       
               c1=!is.numeric(nodeBend)
               if(c1){
                 warning("NOTE: node 'bend' must be provided as numerics!")
                 nodeBend=as.numeric(c(-1,-1))
               } else if(sum(is.na(nodeBend))>0){
                 warning("NOTE: invalid node 'bend': 'NA' found'!")
                 nodeBend=as.numeric(c(-1,-1))
               } else if(sum(nodeBend<0)>0 || sum(nodeBend>100)>0){
                 warning("NOTE: invalid node 'bend' input (options: 0-100%)")
                 nodeBend=as.numeric(c(-1,-1))
               } else {
                 message("** ... node 'bend'") 
                 nodeBend=as.numeric(nodeBend)
               }
             } else {
               nodeBend=as.numeric(c(-1,-1))
             }      
             #nodeSize
             if(!is.null(nodeSize) && length(nodeSize)>0){       
               c1=!is.numeric(nodeSize)
               if(c1){
                 warning("NOTE: node 'size' must be provided as numerics!")
                 nodeSize=as.numeric(c(-1,-1))
               } else if(sum(is.na(nodeSize))>0){
                 warning("NOTE: invalid node 'size': 'NA' found'!")
                 nodeSize=as.numeric(c(-1,-1))
               } else if(sum(nodeSize<0)>0){
                 warning("NOTE: invalid node 'size' input (options: >= 0)")
                 nodeSize=as.numeric(c(-1,-1))
               } else {            
                 message("** ... node 'size'") 
                 nodeSize=as.numeric(nodeSize)
               }
             } else {
               nodeSize=as.numeric(c(-1,-1))
             }        
             #nodeShape
             if(!is.null(nodeShape) && length(nodeShape)>0){       
               c1=!is.character(nodeShape)
               if(c1){
                 warning("NOTE: node 'shape' must be provided as character!")
                 nodeShape=as.character(c('',''))
               } else if(sum(is.na(nodeShape))>0){
                 warning("NOTE: invalid node 'shape': 'NA' found'!")
                 nodeShape=as.character(c('',''))
               } else {
                 message("** ... node 'shape'")
               }  
             }  else {
               nodeShape=as.character(c('',''))
             }                      
             #nodeColor
             if(!is.null(nodeColor) && length(nodeColor)>0 && is.character(nodeColor)){
               if(!is.color(nodeColor)){
                 warning("NOTE: invalid node 'color'")
                 nodeColor=as.character(c('',''))
               } else {
                 nodeColor <- colorRampPalette(nodeColor, alpha=TRUE)(length(nodeColor))
                 message("** ... node 'color'")
               }      
             } else {
               nodeColor=as.character(c('',''))
             }
             #nodeWeight
             if(!is.null(nodeWeight) && length(nodeWeight)>0){       
               c1=!is.numeric(nodeWeight)
               if(c1){
                 warning("NOTE: node 'weight' must be provided as numerics!")
                 nodeWeight=as.numeric(c(0.0,0.0))
               } else if(sum(is.na(nodeWeight))>0){
                 warning("NOTE: invalid node 'weight': 'NA' found'!")
                 nodeWeight=as.numeric(c(-1,-1))
               } else {
                 message("** ... node 'weight'")
                 nodeWeight=as.numeric(nodeWeight) 
               }
             } else {
               nodeWeight=as.numeric(c(0.0,0.0))
             }  
             #nodeLineWidth
             if(!is.null(nodeLineWidth) && length(nodeLineWidth)>0){       
               c1=!is.numeric(nodeLineWidth)
               if(c1){
                 warning("NOTE: node 'line width' must be provided as numerics!")
                 nodeLineWidth=as.numeric(c(-1,-1))
               } else if(sum(is.na(nodeLineWidth))>0){
                 warning("NOTE: invalid node 'line width': 'NA' found'!")
                 nodeLineWidth=as.numeric(c(-1,-1))
               } else if(sum(nodeLineWidth<0)>0){
                 warning("NOTE: invalid node 'line width' input (options: >= 0)")
                 nodeLineWidth=as.numeric(c(-1,-1))
               } else {            
                 message("** ... node 'line width'") 
                 nodeLineWidth=as.numeric(nodeLineWidth)
               }
             } else {
               nodeLineWidth=as.numeric(c(-1,-1))
             }
             #nodeLineColor
             if(!is.null(nodeLineColor) && length(nodeLineColor)>0 && is.character(nodeLineColor)){
               if(!is.color(nodeLineColor)){
                 warning("NOTE: invalid node 'line color'")
                 nodeLineColor=as.character(c('',''))
               } else {
                 nodeLineColor <- colorRampPalette(nodeLineColor, alpha=TRUE)(length(nodeLineColor))
                 message("** ... node 'line color'")
               }      
             } else {
               nodeLineColor=as.character(c('',''))
             }    
             #nodeFontSize
             if(!is.null(nodeFontSize) && length(nodeFontSize)>0){       
               c1=!is.integer(nodeFontSize)
               c2=!is.numeric(nodeFontSize)
               if(c1 && c2){
                 warning("NOTE: node 'font size' must be provided as integer!")
                 nodeFontSize=as.numeric(c(-1,-1))
               } else if(sum(is.na(nodeFontSize))>0){
                 warning("NOTE: invalid node 'font size': 'NA' found'!")
                 nodeFontSize=as.numeric(c(-1,-1))
               } else if(sum(nodeFontSize<0)>0){
                 warning("NOTE: invalid node 'font size' input (options: >= 0)")
                 nodeFontSize=as.numeric(c(-1,-1))
               } else {          
                 message("** ... node 'font size'")
                 nodeFontSize=as.numeric(nodeFontSize) 
               }
             } else {
               nodeFontSize=as.numeric(c(-1,-1))
             }
             #nodeFontColor
             if(!is.null(nodeFontColor) && length(nodeFontColor)>0 && is.character(nodeFontColor)){
               if(!is.color(nodeFontColor)){
                 warning("NOTE: invalid node 'font color'")
                 nodeFontColor=as.character(c('',''))
               } else {
                 nodeFontColor <- colorRampPalette(nodeFontColor, alpha=TRUE)(length(nodeFontColor))
                 message("** ... node 'font color'")
               }      
             } else {
               nodeFontColor=as.character(c('',''))
             }
             
             #Get/set edges attributes (if available)------------------------
             arrowDirection = E(g)$arrowDirection
             arrowLength    = E(g)$arrowLength
             arrowAngle     = E(g)$arrowAngle
             linkType       = E(g)$linkType
             edgeWeight     = E(g)$edgeWeight
             igraphWeight   = E(g)$weight
             edgeWidth      = E(g)$edgeWidth
             edgeColor      = E(g)$edgeColor
             edgeType       = E(g)$edgeType
             
             if(length(igraph::edge_attr_names(g))>0 && igraph::ecount(g)>0){
               message('*** Uploading edge attributes...')   
             }
             
             #set compatibility with igraph!          
             if(is.null(edgeWeight) && !is.null(igraphWeight)) edgeWeight=igraphWeight 
             #correct vectors case only one attr/edge is provided (use dummy values).
             if(nrow(edges)==1){
               edges=rbind(edges,edges)
               if(!is.null(arrowDirection)) arrowDirection=c(arrowDirection,-10)
               if(!is.null(arrowLength))arrowLength=c(arrowLength,-1)
               if(!is.null(arrowAngle))arrowAngle=c(arrowAngle,-1)
               if(!is.null(linkType))linkType=c(linkType,'')
               if(!is.null(edgeWidth)) edgeWidth=c(edgeWidth,-1)
               if(!is.null(edgeColor)) edgeColor=c(edgeColor,'')
               if(!is.null(edgeType)) edgeType=c(edgeType,'')
               if(!is.null(edgeWeight))edgeWeight=c(edgeWeight,0.0)
               if(!is.null(igraphWeight))edgeWeight =c(igraphWeight,0.0)
             }
             
             #arrowDirection                             
             if(!is.null(arrowDirection) && length(arrowDirection)>0){       
               c1=!is.integer(arrowDirection)
               c2=!is.numeric(arrowDirection)
               c3=(sum(arrowDirection< -4)>0 || sum(arrowDirection>4)>0)
               c4=length(arrowDirection)==2 && sum(arrowDirection<0)==1
               if(c1 && c2){
                 warning("NOTE: edge 'direction' must be provided as integers!")
                 arrowDirection=as.numeric(c(-10,-10))
               } else if(sum(is.na(arrowDirection))>0){
                 warning("NOTE: invalid edge 'direction': 'NA' found'!")
                 arrowDirection=as.numeric(c(-10,-10))
               } else if(c3 && !c4){
                 warning("NOTE: invalid 'direction' input (options: (+-) 0, 1, 2, 3 or 4)")
                 arrowDirection=as.numeric(c(-10,-10))
               } else { 
                 arrowDirection=as.numeric(as.integer(arrowDirection))           
                 message("** ... edge 'arrow type/direction'")
               }
             } else {
               arrowDirection=as.numeric(c(-10,-10))
             }
             #arrowLength
             if(!is.null(arrowLength) && length(arrowLength)>0){       
               c1=!is.numeric(arrowLength)
               if(c1){
                 warning("NOTE: arrow 'length' must be provided as numerics!")
                 arrowLength =as.numeric(c(-1,-1))
               } else if(sum(is.na(arrowLength))>0){
                 warning("NOTE: invalid arrow 'length': 'NA' found'!")
                 arrowLength=as.numeric(c(-1,-1))
               } else {            
                 message("** ... arrow 'length'") 
                 arrowLength=as.numeric(arrowLength)
               }
             } else {
               arrowLength=as.numeric(c(-1,-1))
             }   
             #arrowAngle
             if(!is.null(arrowAngle) && length(arrowAngle)>0){       
               c1=!is.numeric(arrowAngle)
               if(c1){
                 warning("NOTE: arrow 'angle' must be provided as numerics!")
                 arrowAngle=as.numeric(c(-1,-1))
               } else if(sum(is.na(arrowAngle))>0){
                 warning("NOTE: invalid arrow 'angle': 'NA' found'!")
                 arrowAngle=as.numeric(c(-1,-1))
               } else {            
                 message("** ... arrow 'angle'") 
                 arrowAngle=as.numeric(arrowAngle)
               }
             } else {
               arrowAngle=as.numeric(c(-1,-1))
             }   
             #edgeWeight
             if(!is.null(edgeWeight) && length(edgeWeight)>0 ){       
               c1=!is.numeric(edgeWeight)
               if(c1){
                 warning("NOTE: edge 'weight' must be provided as numerics!")
                 edgeWeight=as.numeric(c(0.0,0.0))
               } else if(sum(is.na(edgeWeight))>0){
                 warning("NOTE: invalid edge 'weight': 'NA' found'!")
                 edgeWeight=as.numeric(c(0.0,0.0))
               } else {            
                 message("** ... edge 'weight'") 
                 edgeWeight=as.numeric(edgeWeight)
               }
             } else {
               edgeWeight=as.numeric(c(0.0,0.0))
             }      
             #edgeWidth
             if(!is.null(edgeWidth) && length(edgeWidth)>0){       
               c1=!is.numeric(edgeWidth)
               c2=sum(edgeWidth<0)>0
               c3=length(edgeWidth)==2 && sum(edgeWidth<0)==1        
               if(c1){
                 warning("NOTE: edge 'width' must be provided as numerics!")
                 edgeWidth=as.numeric(c(-1,-1))
               } else if(sum(is.na(edgeWidth))>0){
                 warning("NOTE: invalid edge 'width': 'NA' found'!")
                 edgeWidth=as.numeric(c(-1,-1))
               } else if(c2 && !c3){
                 warning("NOTE: invalid edge 'width' input (options: > 0)")
                 edgeWidth=as.numeric(c(-1,-1))
               } else {            
                 message("** ... edge 'width'") 
                 edgeWidth=as.numeric(edgeWidth)
               }
             } else {
               edgeWidth=as.numeric(c(-1,-1))
             }              
             #edgeColor
             if(!is.null(edgeColor) && length(edgeColor)>0 && is.character(edgeColor)){ 
               if(!is.color(edgeColor)){
                 warning("NOTE: invalid edge 'color'")
                 edgeColor=as.character(c('',''))
               } else {
                 edgeColor <- colorRampPalette(edgeColor, alpha=TRUE)(length(edgeColor))
                 message("** ... edge 'color'")
               }      
             } else {
               edgeColor=as.character(c('',''))
             }
             #edgeType
             if(!is.null(edgeType) && length(edgeType)>0){       
               c1=!is.character(edgeType)
               if(c1){
                 warning("NOTE: edge 'type' must be provided as character!")
                 edgeType=as.character(c('',''))
               } else if(sum(is.na(edgeType))>0){
                 warning("NOTE: invalid edge 'type': 'NA' found'!")
                 edgeType=as.character(c('',''))
               } else {            
                 message("** ... edge 'type'") 
               }      
             } else {
               edgeType=as.character(c('',''))
             }
             #linkType
             if(!is.null(linkType) && length(linkType)>0){       
               c1=!is.character(linkType)
               if(c1){
                 warning("NOTE: 'linkType' must be provided as character!")
                 linkType=as.character(c('',''))
               } else if(sum(is.na(linkType))>0){
                 warning("NOTE: invalid 'linkType': 'NA' found'!")
                 linkType=as.character(c('',''))
               } else {            
                 message("** ... edge 'link type'") 
               }      
             } else {
               linkType=as.character(c('',''))
             }    
             #Check nesting condition
             nestref=NULL
             isnp=FALSE
             if(isNested){
               nestpack=nestNodes(obj, nodes, nestImage, isAssigned, isAnchored, gscale=gscale, 
                                  gcoord=NULL, gatt=gatt, theme=theme, getpack=TRUE, 
                                  .zoom=zoom, .callchecks=FALSE)
               np1=nestpack$nodes
               np2=nestpack$status
               np3=nestpack$charAtt
               np4=nestpack$numericAtt
               bl1=is.character(np1) && is.character(np2) && is.character(np3) && is.numeric(np4)
               bl2=length(np1)>=2 && length(np2)==3 && length(np3)==6 && length(np4)==8
               if(bl1 && bl2)isnp=TRUE
             }
             if(isnp){
               isnp="true"  	
             } else {
               np1=as.character(c('',''))
               np2=as.character(c('',''))
               np3=as.character(c('',''))
               np4=as.numeric(c(-1,-1)) 
               isnp="false"
               #usa para passar status a eventuais nodos transformados!
               if(ntransform){
                 np2=c(as.character(nestImage), ifelse(isAnchored,'anchor',''), 
                       ifelse(isAssigned,'assign',''))
               }
             }
             
             #Loading graph...  
             isBrandNew=ifelse(isNested && isAssigned,'true','false')
             layoutOldNodes=ifelse(layoutOldNodes,'true','false')
             ntransform=ifelse(ntransform,'true','false')
             parent=ifelse(is.null(parent),'.$NULL', parent)
             parent=as.character(parent)
             numsuppl=gcoord[1:2]
             charsuppl=c("default",isBrandNew, layoutOldNodes)
             
             if(igraph::ecount(g)>0 && loadEdges){
               #Main call to load nodes and edges
               nestref=.rederexpresspost(obj, 'RedHandler.updateGraphMap', edges[,1],
                                        edges[,2], arrowDirection, edgeWeight, edgeWidth, 
                                        edgeColor, edgeType, arrowLength, arrowAngle, linkType, 
                                        nodes, coordX, coordY, nodeBend, nodeSize, nodeShape, 
                                        nodeColor, nodeWeight, nodeLineWidth, nodeLineColor, 
                                        nodeFontSize, nodeFontColor, nodeAlias, numsuppl, 
                                        charsuppl, np1, np2, np3, np4, isnp, parent, ntransform)      
             } else {
               #Main call to load only nodes
               nestref=.rederexpresspost(obj, 'RedHandler.updateNodeMap', 
                                        nodes, coordX, coordY, nodeBend, nodeSize, nodeShape, 
                                        nodeColor, nodeWeight, nodeLineWidth, nodeLineColor, 
                                        nodeFontSize, nodeFontColor, nodeAlias, numsuppl, 
                                        charsuppl, np1, np2, np3, np4, isnp, 
                                        parent, ntransform)
             }
             
             invisible( updateGraph(obj) )
             
             #Check nesting ref
             if(isNested){
               if(!is.null(nestref)) return(nestref)
             }     
             
           }
)

#-------------------------------------------------------------------------------
setMethod ('getNodes', 'RedPort', 
           function (obj, status="selected", type="node") { 
             if(ping(obj)==0)return(NULL)
             return (.rederpost(obj, 'RedHandler.getNodes', type, status))
           }
)

#-------------------------------------------------------------------------------
setMethod ('getNodeIDs', 'RedPort', 
           function (obj, status="all", type="node") {
             if(ping(obj)==0)return(NULL)
             nodes <- .rederpost(obj, 'RedHandler.getNodeIDs', type, status)
             nodes <- nodes+1 #set index for R!
             return(nodes)
           }
)

#-------------------------------------------------------------------------------
setMethod ('getEdgeIDs', 'RedPort', 
           function (obj, status="all", type="node") {
             if(ping(obj)==0)return(NULL)
             edges <- .rederpost(obj, 'RedHandler.getEdgeIDs', type, status)
             edges <- edges+1 #set index for R!
             return(edges)
           }
)

#-------------------------------------------------------------------------------
setMethod ('getSourceEdgeIDs', 'RedPort', 
           function (obj, status="all", type="node") {
             if(ping(obj)==0)return(NULL)  
             edges <- .rederpost(obj, 'RedHandler.getSourceEdgeIDs', type, status)
             edges <- edges+1 #set index for R!
             return(edges)
           }
)

#-------------------------------------------------------------------------------
setMethod ('getTargetEdgeIDs', 'RedPort', 
           function (obj, status="all", type="node") {
             if(ping(obj)==0)return(NULL)
             edges <- .rederpost(obj, 'RedHandler.getTargetEdgeIDs', type, status)
             edges <- edges+1 #set index for R!
             return(edges)
           }
)

#-------------------------------------------------------------------------------
setMethod ('addNodes', 'RedPort', 
           function (obj, nodes) { 
             if(ping(obj)==0)return(invisible())
             nodes <- as.character(nodes)
             return ( .rederexpresspost(obj, 'RedHandler.addNodes', nodes) )
           }
)

#-------------------------------------------------------------------------------
setMethod ('deleteNodes', 'RedPort', 
           function (obj, nodes) { 
             if(ping(obj)==0)return(invisible())
             nodes=as.character(nodes)
             return (.rederexpresspost(obj, 'RedHandler.deleteNodes', nodes))
           }
)    

#-------------------------------------------------------------------------------
setMethod ('nestNodes', 'RedPort', 
           function (obj, nodes, nestImage ='plain', isAssigned=TRUE, isAnchored=FALSE, 
                     gscale=40, gcoord=NULL, parent=NULL, gatt=list(), 
                     theme=c('tm0','tm1','tm2','tm3','tm4','tm5','tm6'), 
                     getpack=FALSE, .zoom=NULL, .callchecks=TRUE) { 
             
             if(.callchecks){
               if(ping(obj)==0)return(invisible())
             }
             
             #check loaded igraph
             igraph.check()
             
             #Further checks---------------------------------------------------- 
             if(!is.list(gatt)){
               stop("NOTE: 'gatt' must be a list of graph attributes (e.g. gatt$nestColor, gatt$gscale...)!")
             }
             
             # get zoom for themes
             if(!is.numeric(.zoom)){
               if(getpack){
                 .zoom=100
               } else {
                 .zoom <- .rederpost(obj,'RedHandler.getZoom')
                 .zoom <- as.numeric(.zoom)
                 if(is.nan(.zoom)){
                   .zoom=100
                 } else if(.zoom>100){
                   .zoom=100
                 }
                 if(.zoom>100).zoom=100
               }
             } else {
               if(is.nan(.zoom)).zoom=100
             }
             
             if(is.numeric(theme[1])){
               theme=as.integer(theme[1])
               theme=ifelse(theme>=0 && theme<=6, theme, 0)
             } else if(is.character(theme[1])){
               theme=switch(theme[1], tm1=1, tm2=2, tm3=3,tm4=4, tm5=5,tm6=6, 0)
             } else if(is.list(theme)){
               gatt=theme
               if(is.null(theme$theme)){
                 theme=0
               } else {
                 theme=theme$theme
               }
             } else {
               theme=0
             }
             if(theme==1){
               if(is.null(gatt$nestShape))gatt$nestShape='ROUNDED_RECTANGLE'
               if(is.null(gatt$nestColor))gatt$nestColor='#ffffff'
               if(is.null(gatt$nestLineWidth))gatt$nestLineWidth=4*(100/.zoom)
               if(is.null(gatt$nestFontSize))gatt$nestFontSize=24*(100/.zoom)
               if(is.null(gatt$nestFontX))gatt$nestFontX=5
               if(is.null(gatt$nestFontY))gatt$nestFontY=10.8
               if(is.null(gatt$isAssigned))gatt$isAssigned=TRUE
             } else if(theme==2){
               if(is.null(gatt$nestShape))gatt$nestShape='ROUNDED_RECTANGLE'
               if(is.null(gatt$nestColor))gatt$nestColor='#ffffff'
               if(is.null(gatt$nestLineWidth))gatt$nestLineWidth=3*(100/.zoom)
               if(is.null(gatt$nestLineColor))gatt$nestLineColor='#000000'
               if(is.null(gatt$nestLineType))gatt$nestLineType='DOTTED'
               if(is.null(gatt$isAnchored))gatt$isAnchored=TRUE
               if(is.null(gatt$isAssigned))gatt$isAssigned=TRUE
             } else if(theme==3){
               if(is.null(gatt$nestShape))gatt$nestShape='ROUNDED_RECTANGLE'
               if(is.null(gatt$nestColor))gatt$nestColor='#ffffff'
               if(is.null(gatt$nestLineWidth))gatt$nestLineWidth=2*(100/.zoom)
               if(is.null(gatt$nestLineColor))gatt$nestLineColor='#000000'
               if(is.null(gatt$nestLineType))gatt$nestLineType='DOTTED'
               if(is.null(gatt$isAnchored))gatt$isAnchored=TRUE
               if(is.null(gatt$isAssigned))gatt$isAssigned=TRUE
             } else if(theme==4){
               if(is.null(gatt$nestImage))gatt$nestImage='transparent'
               if(is.null(gatt$isAnchored))gatt$isAnchored=TRUE
             } else if(theme==5){
               if(is.null(gatt$nestImage))gatt$nestImage='hide'
               if(is.null(gatt$isAnchored))gatt$isAnchored=TRUE
             } else if(theme==6){
               if(is.null(gatt$nestShape))gatt$nestShape='ROUNDED_RECTANGLE'
               if(is.null(gatt$nestColor))gatt$nestColor='#ffffff'
               if(is.null(gatt$nestLineWidth))gatt$nestLineWidth=4*(100/.zoom)
               if(is.null(gatt$nestFontSize))gatt$nestFontSize=24*(100/.zoom)
               if(is.null(gatt$nestLineColor))gatt$nestLineColor='#000000'
               if(is.null(gatt$nestLineType))gatt$nestLineType='DOTTED'
               if(is.null(gatt$nestFontX))gatt$nestFontX=5
               if(is.null(gatt$nestFontY))gatt$nestFontY=10.8
               if(is.null(gatt$isAnchored))gatt$isAnchored=TRUE
             }
             
             node=as.character(nodes) 	
             status1='plain'
             if(is.character(gatt$nestImage)){
               if(gatt$nestImage=='plain' || gatt$nestImage=='hide' || gatt$nestImage=='transparent')status1=gatt$nestImage[1]
             } else {
               if(is.character(nestImage)){
                 if(nestImage=='plain' || nestImage=='hide' || nestImage=='transparent')status1=nestImage[1]
               }
             } 	
             status2='default'
             if(is.logical(gatt$isAnchored)){
               if(gatt$isAnchored)status2='anchor'
             } else {
               if(is.logical(isAnchored)){
                 if(isAnchored)status2='anchor'
               }
             }
             status3='default'
             if(is.logical(gatt$isAssigned)){
               if(gatt$isAssigned) status3='assign'
             } else {
               if(is.logical(isAssigned)){
                 if(isAssigned) status3='assign'
               }
             }  	  	
             
             if(is.character(gatt$parent) && length(gatt$parent)>0){
               parent=gatt$parent[1]
               nodes=paste(nodes,".$", parent,sep="")
             } else {
               if(is.character(parent) && length(parent)>0){
                 parent=parent[1]
                 nodes=paste(nodes,".$", parent,sep="")
               }
             }
             
             #Get string attributes 
             charAtt=rep("<$$>",6)
             if(!is.null(gatt) && !getpack){
               message('*** Uploading nest attributes ...')
             }
             #Nest aliases
             if(is.character(gatt$nestAlias) && length(gatt$nestAlias)>0){
               if(!getpack)message("** ... nest 'alias'")
               charAtt[1]=gatt$nestAlias[1]
             } else if(!is.null(gatt$nestAlias)){
               warning("NOTE: nest 'alias' must be provided as character!")
             }
             #Nest shape
             if(is.character(gatt$nestShape)){
               gatt$nestShape = gatt$nestShape[1]
               if(is.na(gatt$nestShape)){
                 warning("NOTE: invalid nest 'shape': 'NA' found'!")
               } else {            
                 if(!getpack)message("** ... nest 'shape'") 
                 charAtt[2]=gatt$nestShape
               }      
             } else if(!is.null(gatt$nestShape)){
               warning("NOTE: nest 'shape' must be provided as character!")
             } 	
             #Nest line type
             if(is.character(gatt$nestLineType)){
               gatt$nestLineType = gatt$nestLineType[1]
               if(is.na(gatt$nestLineType)){
                 warning("NOTE: invalid nest 'line type': 'NA' found'!")
               } else {            
                 if(!getpack)message("** ... nest 'line type'") 
                 charAtt[3]=gatt$nestLineType
               }      
             } else if(!is.null(gatt$nestLineType)){ 
               warning("NOTE: nest 'line type' must be provided as character!")
             }	
             #Nest color
             if(is.character(gatt$nestColor)){
               gatt$nestColor <- gatt$nestColor[1]
               if(!is.color(gatt$nestColor)){
                 warning("NOTE: invalid nest 'color'")
               } else {            
                 if(!getpack)message("** ... nest 'color'")
                 charAtt[4]=colorRampPalette(gatt$nestColor, alpha=TRUE)(1)
               }
             } else if(!is.null(gatt$nestColor)){ 
               warning("NOTE: nest 'color' must be provided as hexadecimal.")
             }
             #Nest line color
             if(is.character(gatt$nestLineColor)){
               gatt$nestLineColor <- gatt$nestLineColor[1]
               if(!is.color(gatt$nestLineColor)){
                 warning("NOTE: invalid nest 'line color'")
               } else {            
                 if(!getpack)message("** ... nest 'line color'") 
                 charAtt[5]=colorRampPalette(gatt$nestLineColor, alpha=TRUE)(1)
               }      
             } else if(!is.null(gatt$nestLineColor)){ 
               warning("NOTE: nest 'line color' must be provided as character (hexadecimal)!")
             }
             #Nest font color
             if(is.character(gatt$nestFontColor)){
               gatt$nestFontColor[1] <-gatt$nestFontColor
               if(!is.color(gatt$nestFontColor)){
                 warning("NOTE: invalid nest 'font color'")
               } else {            
                 if(!getpack)message("** ... nest 'line color'")
                 charAtt[6]=colorRampPalette(gatt$nestFontColor, alpha=TRUE)(1)
               }
             } else if(!is.null(gatt$nestFontColor)){ 
               warning("NOTE: nest 'font color' must be provided as character (hexadecimal)!")
             }
             
             #Get numerics attributes   		 	
             numericAtt=c(-8,-8,-1,-1,-1,-1, 909192, 909192)
             #Nest font coords.
             if(is.numeric(gatt$nestFontX) && is.numeric(gatt$nestFontY)){
               gatt$nestFontX = gatt$nestFontX[1]
               gatt$nestFontY = gatt$nestFontY[1]
               if(is.na(gatt$nestFontX) || is.na(gatt$nestFontY) ){
                 warning("NOTE: invalid nest 'nestFontX' or 'nestFontY': 'NA' found'!")
               } else {
                 if(!getpack)message("** ... nest font 'coords'")
                 numericAtt[1]=gatt$nestFontX
                 numericAtt[2]=gatt$nestFontY
               }      
             }  
             #Nest font size.
             if(is.numeric(gatt$nestFontSize)){
               gatt$nestFontSize = gatt$nestFontSize[1]
               if(is.na(gatt$nestFontSize) ){
                 warning("NOTE: invalid nest 'font size': 'NA' found'!")              
               } else if(gatt$nestFontSize<0){
                 warning("NOTE: invalid nest 'font size' (options: >= 0)")
               } else {
                 if(!getpack)message("** ... nest font 'size'") 
                 numericAtt[3]=gatt$nestFontSize
               }
             }	  		
             #Nest line width
             if(is.numeric(gatt$nestLineWidth)){
               gatt$nestLineWidth = gatt$nestLineWidth[1]
               if(is.na(gatt$nestLineWidth) ){
                 warning("NOTE: invalid nest 'line width': 'NA' found'!")              
               } else if(gatt$nestLineWidth<0){
                 warning("NOTE: invalid nest 'line width' (options: >= 0)")
               } else {
                 if(!getpack)message("** ... nest 'line width'") 
                 numericAtt[4]=gatt$nestLineWidth
               }
             }
             #Nest size.
             if(is.numeric(gatt$nestSize)){
               gatt$nestSize = gatt$nestSize[1]
               if(is.na(gatt$nestSize) ){
                 warning("NOTE: invalid nest 'size': 'NA' found'!")              
               } else if(gatt$nestSize<0){
                 warning("NOTE: invalid nest 'size' (options: >= 0)")
               } else {
                 if(!getpack)message("** ... nest 'size'") 
                 numericAtt[5]=gatt$nestSize
               }
             }	
             #Nest gscale
             if(!is.null(gscale) && is.null(gatt$nestSize)){	
               if(is.numeric(gscale)){
                 if(gscale>=1)numericAtt[6]=gscale[1]	
               } else if(is.numeric(gatt$gscale)){
                 gatt$gscale = gatt$gscale[1]
                 if(is.na(gatt$gscale) ){
                   warning("NOTE: invalid nest 'gscale': 'NA' found'!")              
                 } else if(gatt$gscale <0){
                   warning("NOTE: invalid nest 'gscale' (options: > 0)")
                 } else {
                   if(!getpack)message("** ... nest 'gscale'") 
                   numericAtt[6]=gatt$gscale
                 }
               } 	
             }
             #Nest gcoord	
             if(!is.null(gcoord)){
               if(is.numeric(gcoord) && length(gcoord)==2){
                 numericAtt[7]= gcoord[1]
                 numericAtt[8]= gcoord[2]	
               } else if(is.numeric(gatt$gcoord) && length(gcoord)==2){
                 if(sum(is.na(gatt$gcoord))>0 ){
                   warning("NOTE: invalid nest 'gcoord': 'NA' found'!")              
                 } else {
                   if(!getpack)message("** ... nest 'gcoord'") 
                   numericAtt[7]=gatt$gcoord[1]
                   numericAtt[8]=gatt$gcoord[2]
                 }
               } 	
             }
             if(getpack){
               nestpack=list(nodes=nodes, status=c(status1,status2,status3), charAtt=charAtt, 
                             numericAtt=numericAtt, nestmap=length(nodes))
               return(nestpack)
             } else {
               ref=.rederexpresspost(obj, 'RedHandler.nestexpress', nodes, c(status1,status2,status3), 
                                    charAtt, numericAtt )      
               invisible( updateGraph(obj) )
               return(ref)		
             }   
             
           }
)

#-------------------------------------------------------------------------------
setMethod ('updateContainerSize', 'RedPort', 
           function (obj) { 
             if(ping(obj)==0)return(invisible())
             return (.rederpost(obj, 'RedHandler.updateContainerSize'))
           }
)

#-------------------------------------------------------------------------------
setMethod ('mergeOutEdges', 'RedPort', 
           function (obj, nlevels=2, rescale=TRUE, lb=NA, ub=NA, nlev=NULL) { 
             
             if(ping(obj)==0)return(invisible())
             
             #deprecated parameter
             if(!is.null(nlev)){
               warning("'nlev' attribute is deprecated; please use 'nlevels'.")
               nlevels <- nlev
             }
             
             if(!is.numeric(nlevels) && !is.integer(nlevels)) nlevels=2
             if(is.nan(nlevels)) nlevels=2
             if(is.logical(rescale)){
               rescale=ifelse(rescale,'true','false')
             } else {
               rescale='true'
             }
             lb=lb[1]; ub=ub[1]
             if(rescale!='true' || !is.numeric(lb) || !is.numeric(ub)){
               lb=0; ub=0
             } else {
               lb=max(lb,0)
               ub=max(ub,0)
             }
             for(i in 1:nlevels){
               res=.rederexpresspost(obj, 'RedHandler.mergeContainerOutEdges', rescale, lb, ub)           
             }
             res
           }
)

#-------------------------------------------------------------------------------
setMethod ('getContainerComponets', 'RedPort', 
           function (obj, container) { 
             if(ping(obj)==0)return(invisible())
             container=as.character(container)
             return (.rederexpresspost(obj, 'RedHandler.getContainerComponets', container))
           }
)

#-------------------------------------------------------------------------------
setMethod ('mergeNodes', 'RedPort', 
           function (obj, nodes) { 
             if(ping(obj)==0)return(invisible())
             node=as.character(nodes)
             return (.rederexpresspost(obj, 'RedHandler.mergeNodes', nodes))
           }
)

#-------------------------------------------------------------------------------
setMethod ('getEdges', 'RedPort', 
           function (obj, status="selected", type="node") { 
             if(ping(obj)==0)return(NULL)
             return (.rederpost(obj, 'RedHandler.getEdges', type, status))
           }
)

#-------------------------------------------------------------------------------
setMethod ('setArrowDirection', 'RedPort', 
           function (obj, nodeA, nodeB, direction) { 
             if(ping(obj)==0)return(invisible())
             c1=!is.integer(direction)
             c2=!is.numeric(direction)
             if(c1 && c2){
               stop("Arrow 'direction' must be provided as integers!")
             }
             if(sum(is.na(direction))>0){
               stop("Invalid arrow 'direction': 'NA' found'!")
             }
             if(sum(direction<0)>0 || sum(direction>3)>0){
               stop("Invalid arrow 'direction' (options: 0, 1, 2 or 3)")
             }
             nodeA=as.character(nodeA)
             nodeB=as.character(nodeB)  
             direction=as.numeric(direction)  
             return (.rederexpresspost(obj, 'RedHandler.setArrowDirection', 
                             nodeA, nodeB, direction))
           }
)

#-------------------------------------------------------------------------------
setMethod ('addEdges', 'RedPort', 
           function (obj, edges) {
             if(ping(obj)==0)return(invisible())
             c1=is.data.frame(edges) && ncol(edges)==2
             c2=is.character(edges)
             if(!c1 && !c2 ){
               stop("Edges must be provided as a 'vector' or 'data.frame' object.")
             }
             if(c1){
               edges[,1] <- as.character(edges[,1])
               edges[,2] <- as.character(edges[,2])
               edges <- as.character(t(edges))
             }   
             return(.rederexpresspost(obj, 'RedHandler.addEdges', edges))
           }
)

#-------------------------------------------------------------------------------
setMethod ('deleteEdges', 'RedPort', 
           function (obj, edges) {  
             if(ping(obj)==0)return(invisible())
             c1=is.data.frame(edges) && ncol(edges)==2
             c2=is.character(edges)
             if(!c1 && !c2 ){
               stop("Edges must be provided as a 'vector' or 'data.frame' object.")
             }
             if(c1){
               edges[,1] <- as.character(edges[,1])
               edges[,2] <- as.character(edges[,2])
               edges <- as.character(t(edges))
             }
             return (.rederexpresspost(obj, 'RedHandler.deleteEdges', edges))
           }
)

#-------------------------------------------------------------------------------
setMethod ('addEdgeBetweenContainers', 'RedPort', 
           function (obj, containerA, containerB) { 
             if(ping(obj)==0)return(invisible())
             containerA=as.character(containerA)
             containerB=as.character(containerB)   
             return (.rederexpresspost(obj, 'RedHandler.addEdgeBetweenContainers', 
                              containerA, containerB))
           }
)

#-------------------------------------------------------------------------------
setMethod ('selectEdges', 'RedPort', 
           function (obj, edges) {
             if(ping(obj)==0)return(invisible())
             c1=is.data.frame(edges) && ncol(edges)==2
             c2=is.character(edges)
             if(!c1 && !c2 ){
               stop("Edges must be provided as a 'vector' or 'data.frame' object.")
             }
             if(c1){
               edges[,1] <- as.character(edges[,1])
               edges[,2] <- as.character(edges[,2])
               edges <- as.character(t(edges))
             }
             deSelectEdges(obj)  #deselect all edges previously to the call!    
             invisible(.rederexpresspost(obj, 'RedHandler.selectEdges', edges))
           }
)

#-------------------------------------------------------------------------------
setMethod ('selectNodes', 'RedPort', 
           function (obj, nodes, anchor=FALSE, nt=NULL) { 
             if(ping(obj)==0)return(invisible())
             nodes=as.character(nodes)
             nt=ifelse(is.null(nt[1]),"",as.character(nt)[1])
             anchor=ifelse(anchor,"true","false")
             deSelectNodes(obj)#deselect all nodes previously to the call!
             if(length(nodes)>1){
               invisible(.rederexpresspost(obj, 'RedHandler.selectNodeList', 
                                            nodes, anchor, nt))
             } else {
               invisible(.rederexpresspost(obj, 'RedHandler.selectNodeString', 
                                           nodes, anchor, nt))
             }
           }
)

#-------------------------------------------------------------------------------
setMethod ('selectAllEdges', 'RedPort', 
           function (obj) {   
             if(ping(obj)==0)return(invisible())
             invisible (.rederpost(obj, 'RedHandler.selectAllEdges'))
           }
)

#-------------------------------------------------------------------------------
setMethod ('selectAllNodes', 'RedPort', 
           function (obj) {   
             if(ping(obj)==0)return(invisible())
             invisible (.rederpost(obj, 'RedHandler.selectAllNodes'))
           }
)

#-------------------------------------------------------------------------------
setMethod ('selectGraph', 'RedPort', 
           function (obj) {   
             if(ping(obj)==0)return(invisible())
             invisible (.rederpost(obj, 'RedHandler.selectGraph'))
           }
)

#-------------------------------------------------------------------------------
setMethod ('deSelectEdges', 'RedPort', 
           function (obj) {   
             if(ping(obj)==0)return(invisible())
             invisible (.rederpost(obj, 'RedHandler.deSelectEdges'))
           }
)

#-------------------------------------------------------------------------------
setMethod ('deSelectNodes', 'RedPort', 
           function (obj) {   
             if(ping(obj)==0)return(invisible())
             invisible (.rederpost(obj, 'RedHandler.deSelectNodes'))
           }
)

#-------------------------------------------------------------------------------
setMethod ('deSelectGraph', 'RedPort', 
           function (obj) {   
             if(ping(obj)==0)return(invisible())
             invisible (.rederpost(obj, 'RedHandler.deSelectGraph'))
           }
)

#-------------------------------------------------------------------------------
setMethod ('deleteSelectedEdges', 'RedPort', 
           function (obj) {   
             if(ping(obj)==0)return(invisible())
             invisible (.rederpost(obj, 'RedHandler.deleteSelectedEdges'))
           }
)

#-------------------------------------------------------------------------------
setMethod ('deleteSelectedNodes', 'RedPort', 
           function (obj) {
             if(ping(obj)==0)return(invisible())
             invisible (.rederpost(obj, 'RedHandler.deleteSelectedNodes'))
           }
)

#-------------------------------------------------------------------------------
setMethod ('isRelaxActive', 'RedPort', 
           function (obj) {   
             if(ping(obj)==0)return(invisible())
             return (.rederpost(obj, 'RedHandler.isRelaxActive'))
           }
)

#-------------------------------------------------------------------------------
setMethod ('relax', 'RedPort', 
           function (obj,p1=100,p2=100,p3=100,p4=100,p5=100,p6=10,p7=10,p8=100,
                     p9=10, ps=FALSE) {
             #ps: deprecated
             if(ping(obj)==0)return(invisible())
             if(!is.numeric(p1) || length(p1)==0)p1=100;p1=p1[1]
             if(!is.numeric(p2) || length(p2)==0)p2=100;p2=p2[1]
             if(!is.numeric(p3) || length(p3)==0)p3=100;p3=p3[1] 		  		
             if(!is.numeric(p4) || length(p4)==0)p4=100;p4=p4[1]
             if(!is.numeric(p5) || length(p5)==0)p5=100;p5=p5[1]  		
             if(!is.numeric(p6) || length(p6)==0)p6=100;p6=p6[1]  		
             if(!is.numeric(p7) || length(p7)==0)p7=10;p7=p7[1]
             if(!is.numeric(p8) || length(p8)==0)p8=10;p8=p8[1]
             if(!is.numeric(p9) || length(p9)==0)p9=1;p9=p9[1]
             return (.rederexpresspost(obj, 'RedHandler.setRelax',
                                       p1,p2,p3,p4,p5,p6,p7,p8,p9))
           }
)

#-------------------------------------------------------------------------------
setMethod ('nesthc', 'RedPort', 
           function(obj, hc, cutlevel=2, metric=c("rootdist","leafdist","height"), 
                    nmemb=2, nlev=2, grid=c(2,3), gridScale=75, gscale=c(30,75,45), 
                    isAssigned=FALSE, isAnchored=TRUE, theme='tm6', nlinewidth=10, 
                    nfontsz=60, col=NULL, plothc=TRUE, plotbox=TRUE, cex=0.6, 
                    xlab="Nodes", ylab="Height", main="Hierarchical Network",
                    labels=NULL, lwd=1,...){
             
             if(ping(obj)==0)return(invisible())
             
             #check loaded igraph
             igraph.check()
             
             #check hclust object-----------------------------------------------
             if(class(hc)!="hclust"){
               stop("Not a hclust object!")
             }
             hc$height=hc$height/max(hc$height)
             
             #check args--------------------------------------------------------
             
             if(is.character(metric)){
               tp=switch(metric[1], rootdist=1, leafdist=2, height=3, 1)
             } else if(is.numeric(metric)){
               tp=metric[1]
               tp=ifelse(tp>0 && tp<=3,tp, 1)
             } else {
               tp=1
             }
             metric=c("rootdist","leafdist","height")[tp]
             nmemb=max(2,nmemb)
             
             #further checks----------------------------------------------------
             if(!is.numeric(cutlevel))cutlevel=2
             cutlevel=cutlevel[1]
             if(!is.numeric(nmemb))nmemb=2
             nmemb=max(2,nmemb[1])
             if(!is.numeric(grid))grid=c(2,3)
             if(length(grid)==1)grid=c(2,3)
             gridScale=gridScale[1]
             if(!is.numeric(gridScale))gridScale=75
             if(gridScale<1)gridScale=1
             s1=!is.numeric(gscale)
             s2=is.null(gscale)
             s3=sum(is.na(gscale))>0
             if(s1 || s2 || s3){
               gscale = c(30,75,45)
             }		
             
             if(is.numeric(grid) && length(grid)>1){
               gridRows=ifelse(!is.na(grid[1]),grid[1],2)
               gridCols=ifelse(!is.na(grid[2]),grid[2],3)
             } else {
               gridRows=2
               gridCols=3			
             }
             
             # map tree
             tm=treemap(hc)
             nn=length(tm$nest)
             
             #compute dist to root and select nested series	
             val=tm[[metric]]
             if(metric=="rootdist"){
               selec=as.numeric(val>=cutlevel)
             } else {
               selec=as.numeric(val<=cutlevel)
             }
             tp=tm$parent
             td=selec[tp]
             td[is.na(td)]=0
             nrtdist=td
             while(sum(td)>0){
               tp=tm$parent[tp]
               td=selec[tp]
               td[is.na(td)]=0
               nrtdist=nrtdist+td
             }
             selec=nrtdist+selec	
             selec[selec>nlev]=0
             tedges=c(1:length(selec))[selec!=0]
             # remap parents (e.g. after removing tree nodes via pvclust)
             tp=tm$parent
             td=tp[tedges]
             tdd=tp[tedges]
             idx=!td%in%tedges
             while(sum(idx)>0){
               td[idx]=tp[td[idx]]
               idx=!td%in%tedges
               idx[is.na(td)]=FALSE
               #td[is.na(td)]=tp[tedges][is.na(td)]	
               tdd[is.na(td)]=tp[tedges][is.na(td)]				
             }
             newparent=rep(0,nn)
             newparent[tedges]=tdd
             newnnest=rep(0,length(newparent))
             for(i in 1:length(newnnest)){
               newnnest[i]=sum(newparent==i)
             }
             nestcount=sum(selec!=0)
             
             #estimate grid
             if(!is.numeric(gridRows)){gridRows=NULL}else{gridRows=gridRows[1]}
             zoom=NULL
             if(is.numeric(gridScale)){
               gridScale=gridScale[1]
               #set gridScale to zoom
               if(gridScale>100)gridScale=100
               if(gridScale<0)gridScale=0
               zoom=100-gridScale   	
             }
             #get a basic layout just for graphs' first view
             if(is.null(gridRows)){
               gbasic=igraph::make_empty_graph(n=nestcount,directed=FALSE)
               layout=igraph::norm_coords(igraph::layout.circle(gbasic), xmin = 25, xmax=75, ymin=25, ymax=75)
             } else {
               bin=100/(gridCols+1)
               xgrid=c(1:gridCols)*bin
               bin=100/(gridRows +1)
               estimatedRows=as.integer(nestcount/gridCols)+1
               ygrid=c(1: estimatedRows)*bin
               layout=cbind(x=xgrid,y=ygrid[1])
               if(estimatedRows>1){
                 for(i in 2: estimatedRows){
                   lt=cbind(x=xgrid,y=ygrid[i])
                   layout=rbind(layout, lt)
                 }
               }
             }	
             
             # ' update="default" ' forces keeping old node coords and not to add new containers!
             if(is.null(update) || !is.character(update))update=NULL
             # internal function (locks DragAndZoon interactivity while sending the subgraph list to the data bank)
             invisible(.rederpost(obj,'RedHandler.lockDragAndZoom'))
             if(!is.null(zoom))invisible( .rederexpresspost(obj, 'RedHandler.setZoom',zoom) )
             checknd=getGraph(obj, status="all", attribs="minimal")
             checknd=V(checknd)$name        
             
             # set lab colors
             if(is.null(col)){
               col=c("darkred","red","orange","darkgreen","cyan","blue","darkblue")
               col=colorRampPalette(colors=col, alpha=TRUE)(nestcount)
             } else {
               if(length(col)<2)col=c(col,col)
               col=colorRampPalette(colors=col)(nestcount)
             }
             
             #add tree ----------------------------------------------------------
             gs0=gscale[1]
             gs1=gscale[2]
             gs2=gscale[3]
             gc1=c(58,58)
             gc2=c(70,70)
             gc3=c(30,30)
             stats=data.frame()
             nestpack=list()
             nestcount=1
             nid=rep(NA,nn)
             returnIdList<-list()
             k=1
             for(i in rev(tedges)){			
               nodes=tm$nest[[i]]
               if(sum(nodes%in%checknd)>=nmemb){
                 pt=newparent[i]
                 if(!is.na(nid[pt]) && isAssigned){
                   nodes=paste(nodes,".$", nid[pt],sep="")
                 }
                 if(is.na(nid[pt])){	
                   gs=gs0
                   gc=c(layout[k,1],layout[k,2])
                   k=k+1				
                 } else if(newnnest[pt]<2){
                   gs=gs1
                   gc=gc1  				
                 } else if(newnnest[pt]==2){
                   gs=gs2
                   gc=gc2
                   newnnest[pt]=3
                 } else if(newnnest[pt]==3) {
                   gs=gs2
                   gc=gc3  				
                 }
                 #scale nest.line.width by n. levels (just to get a better image!) 
                 gatt=list()
                 gatt$nestLineWidth=(nlinewidth/2)+((nlinewidth/2)*(1/max(1,selec[i])))
                 gatt$nestFontSize=(nfontsz/2)+((nfontsz/2)*(1/max(1,selec[i])))
                 #gatt$nestFontColor=col[tedges==i]
                 gatt$nestLineColor=col[tedges==i]
                 gatt$nestAlias=paste("NT",nestcount,sep="")
                 nid[i]=gatt$nestAlias
                 #send nested nodes!
                 if(isAssigned){
                   nestNodes(obj, nodes, nestImage='plain', isAssigned=isAssigned, isAnchored=isAnchored, 
                             gscale=gs, gcoord=gc, gatt=gatt, theme=theme)
                 } else {
                   nestpack[[nestcount]]=nestNodes(obj, nodes, nestImage='plain', isAssigned=isAssigned, 
                                                   isAnchored=isAnchored, gscale=gs, gcoord=gc, gatt=gatt, theme=theme, 
                                                   getpack=TRUE, .callchecks=FALSE)
                   nestcount=nestcount+1
                 }
                 returnIdList[[nid[i]]] <- nodes
                 stats=rbind(stats, data.frame(nid=nid[i],did=i,nest.size=length(nodes),
                                               leafdist=tm$leafdist[i],root.dist=tm$rootdist[i],height=tm$height[i],
                                               nestroot.dist=selec[i],stringsAsFactors=FALSE))		
               } else {
                 selec[i]=0 #remove container due to n. members<threashold!!
               }
             }
             if(length(nestpack)>0){
               nodes=nestpack[[1]]$nodes
               status=nestpack[[1]]$status
               charAtt=nestpack[[1]]$charAtt
               numAtt=nestpack[[1]]$numericAtt
               nestmap=nestpack[[1]]$nestmap 
               if(length(nestpack)>1){ 	  	
                 for(i in 2:length(nestpack)){
                   nodes=c(nodes,nestpack[[i]]$nodes)
                   status=c(status,nestpack[[i]]$status)
                   charAtt=c(charAtt,nestpack[[i]]$charAtt)
                   numAtt=c(numAtt,nestpack[[i]]$numericAtt)
                   nestmap=c(nestmap,nestpack[[i]]$nestmap)    	  		
                 }
               }
               message('*** Uploading nest hclust...')
               .rederexpresspost(obj,'RedHandler.nestpackexpress', nodes, status, charAtt, numAtt, nestmap)
               invisible( updateGraph(obj) )
             }
             
             #Internal function (unlocks DragAndZoon interactivity after send subgraph list)
             invisible(.rederpost(obj,'RedHandler.unLockDragAndZoom'))
             
             if(plothc){
               plot(x=hc, xlab=xlab, ylab=ylab, cex=cex, sub="", main=main, labels=labels, lwd=lwd,...)
               if(sum(selec>0)>0){
                 usr=par()$usr; wid=usr[4]-usr[3]
                 cex=cex*2
                 cex1=cex-(cex/2)
                 cex2=cex/2
                 cex=cex1+cex2*1/stats$nestroot.dist
                 text(x=tm$xyaxis[stats$did,1], y=tm$xyaxis[stats$did,2] + 0.02 * wid, 
                      labels=stats$nid, pos=4, offset=.3, col=rev(col), cex=cex, lwd=lwd,...)
                 if(plotbox){
                   comp=tm$compids
                   order=hc$order
                   xwd=usr[2]-usr[1]
                   ywd=usr[4]-usr[3]
                   cin=par()$cin	
                   for(i in rev(tedges)){
                     if(selec[i]>0){ #em funcao do corte para n.nodos
                       mi=comp[[i]]
                       ma=match(mi, order)
                       xl=min(ma)
                       xr=max(ma)
                       yt=hc$height[i]
                       yb=usr[3]
                       mx=xwd / length(comp) / 3
                       my=ywd / 200
                       nd=stats[stats$did==i,"nestroot.dist"]
                       rect(xleft=xl - mx, ybottom=yb + my, xright=xr + mx, ytop=yt + my, 
                            border=col[tedges==i], shade=NULL, lwd=lwd*2*(1/nd),...)
                     }
                   }
                 }
               }
             }
             #return final ids
             res <- getGraph(obj,type="all", attribs="all")
             #---
             idx <- match(names(returnIdList),V(res)$nodeAlias)
             names(returnIdList) <- V(res)$name[idx]
             idx <- match(stats$nid,V(res)$nodeAlias)
             stats$rid <- V(res)$name[idx]
             stats <- stats[,c("rid","nid","did","nest.size")]
             colnames(stats) <- c("nestID","nestAlias","hcID","nestSize")
             invisible(list(nests=returnIdList, nestID=stats))
           }
)

#-------------------------------------------------------------------------------
setMethod ('addLegend.color', 'RedPort', 
           function (obj, colvec, type="node", labvec=NULL, position=NULL, 
                     dxborder=NULL, dyborder=NULL, vertical=NULL, 
                     ftsize=NULL, title=NULL, dxtitle=NULL, 
                     size=NULL, bend=NULL) {
             
             if(ping(obj)==0)return(invisible())
             
             #check loaded igraph
             igraph.check()
             
             # checks--------------------------------------------------------- 
             
             if(!is.character(type))type="nodecolor"
             type=switch(type, node="nodecolor", edge="edgecolor", "nodecolor")
             
             #Check if igraph object------------------------------------------
             if(igraph::is_igraph(colvec)){
               if(type=="nodecolor"){
                 if(!is.null(G(colvec,"legNodeColor")$scale)){
                   if(!is.null(G(colvec,"legNodeColor")$legend) && is.null(labvec)){
                     labvec=G(colvec,"legNodeColor")$legend
                     if(is.null(title))title=G(colvec,"legNodeColor")$title
                   }
                   colvec=G(colvec,"legNodeColor")$scale
                 } else {
                   stop("NOTE: there is no valid 'legNodeColor' legend information for this igraph object!!")
                 }
               } else {
                 if(!is.null(G(colvec,"legEdgeColor")$scale)){
                   if(!is.null(G(colvec,"legEdgeColor")$legend) && is.null(labvec)){
                     labvec=G(colvec,"legEdgeColor")$legend
                     if(is.null(title))title=G(colvec,"legEdgeColor")$title
                   }
                   colvec=G(colvec,"legEdgeColor")$scale
                 } else {
                   stop("NOTE: there is no valid 'legEdgeColor' legend information for this igraph object!!")
                 }			
               }
             }
             
             # colors
             if(!is.null(colvec) && length(colvec)>0 && is.character(colvec)){
               if(!is.color(colvec)){
                 stop("NOTE: invalid colors; 'colvec' must be provided 
                      as a vector of hexadecimal or valid R colors.")
               } else {
                 colvec <- colorRampPalette(colvec, alpha=TRUE)(length(colvec))
               }      
             } else {
               stop("NOTE: 'colvec' object is not valid.")
             }
             
             # label vec
             if(is.null(labvec))labvec=letters[c(1:length(colvec))]
             if(length(labvec)!=length(colvec)){
               stop("NOTE: 'labvec' and 'colvec' must have the same length!")
             }
             labvec=as.character(labvec)
             
             # further args
             if(!is.null(position) && !is.character(position))position=NULL
             position=position[1]
             if(!is.null(dxborder) && !is.numeric(dxborder))dxborder=NULL
             dxborder=dxborder[1]
             if(!is.null(dyborder) && !is.numeric(dyborder))dyborder=NULL
             dyborder=dyborder[1]
             if(!is.null(vertical) && !is.logical(vertical))vertical=NULL
             vertical=vertical[1]
             if(!is.null(ftsize) && !is.numeric(ftsize))ftsize=NULL
             ftsize=ftsize[1]
             if(!is.null(title) && !is.character(title))title=NULL
             title=title[1]	
             if(!is.null(dxtitle) && !is.numeric(dxtitle))dxtitle=NULL
             dxtitle=dxtitle[1]
             if(!is.null(size) && !is.numeric(size))size=NULL
             size=size[1]	
             if(!is.null(bend) && !is.numeric(bend))bend=NULL
             bend=bend[1]	
             
             if(!is.null(position)){
               position=switch(position, bottomright="bottomRight", 
                               bottomleft="bottomLeft", topright="topRight", topleft="topLeft", NULL)
             }	
             
             # default settings
             if(type=="nodecolor"){
               if(is.null(position))position="topRight"
               if(is.null(dxborder))dxborder=5
               if(is.null(dyborder))dyborder=5
               if(is.null(vertical))vertical=FALSE
               if(is.null(ftsize))ftsize=12
               if(is.null(title))title="nodecolorscale"
               if(is.null(dxtitle))dxtitle=5
               if(is.null(size))size=20
               if(is.null(bend))bend=0.85	
             } else {
               if(is.null(position))position="topRight"
               if(is.null(dxborder))dxborder=5
               if(is.null(dyborder))dyborder=120
               if(is.null(vertical))vertical=FALSE
               if(is.null(ftsize))ftsize=12
               if(is.null(title))title="edgecolorscale"
               if(is.null(dxtitle))dxtitle=5
               if(is.null(size))size=20
               if(is.null(bend))bend=0.85
             }
             bend=as.integer(bend*100)
             bend=min(max(bend,0),100)
             
             # reverse values for vertical
             if(vertical){
               colvec <- rev(colvec)
               labvec <- rev(labvec)
             }

             # set logical
             vertical=ifelse(vertical,"true","false")
             
             invisible( .rederexpresspost(obj, 'RedHandler.addLegendColor', 
                                          colvec, labvec, size, bend, ftsize, 
                                          title, dxtitle, position, dxborder, 
                                          dyborder, vertical, type ) )
             
           }
)

#-------------------------------------------------------------------------------
setMethod ('addLegend.size', 'RedPort', 
           function (obj, sizevec, type="node", labvec=NULL, colvec=NULL, 
                     position=NULL, dxborder=NULL, dyborder=NULL, vertical=NULL, 
                     ftsize=NULL, title=NULL, dxtitle=NULL, intersp=NULL, edgelen=NULL) {
             
             if(ping(obj)==0)return(invisible())
             
             #check loaded igraph
             igraph.check()
             
             # checks---------------------------------------------------- 
             if(!is.character(type))type="nodesize"
             type=switch(type, node="nodesize", edge="edgewidth", "nodesize")
             
             #Check if igraph object------------------------------------------
             if(igraph::is_igraph(sizevec)){
               if(type=="nodesize"){
                 if(!is.null(G(sizevec,"legNodeSize")$scale)){
                   if(!is.null(G(sizevec,"legNodeSize")$legend) && is.null(labvec)){
                     labvec=G(sizevec,"legNodeSize")$legend
                     if(is.null(title))title=G(sizevec,"legNodeSize")$title
                   }
                   sizevec=G(sizevec,"legNodeSize")$scale
                 } else {
                   stop("NOTE: there is no valid 'legNodeSize' legend information for this igraph object!!")
                 }
               } else {
                 if(!is.null(G(sizevec,"legEdgeWidth")$scale)){
                   if(!is.null(G(sizevec,"legEdgeWidth")$legend) && is.null(labvec)){
                     labvec=G(sizevec,"legEdgeWidth")$legend
                     if(is.null(title))title=G(sizevec,"legEdgeWidth")$title
                   }
                   sizevec=G(sizevec,"legEdgeWidth")$scale
                 } else {
                   stop("NOTE: there is no valid 'legEdgeWidth' legend information for this igraph object!!")
                 }			
               }
             }
             
             #size vec
             if(!is.null(sizevec) && length(sizevec)>1){       
               c1=!is.numeric(sizevec)
               if(c1){
                 stop("NOTE: 'size' must be provided as numerics!")
               } else if(sum(is.na(sizevec))>0){
                 stop("NOTE: invalid 'size': 'NA' found'!")
               } else if(sum(sizevec<0)>0){
                 stop("NOTE: invalid node 'size' input (options: >= 0)")
               } else {            
                 sizevec=as.numeric(sizevec)
               }
             } else if(is.null(sizevec)){
               type="null"
               sizevec=c(0,0)	
               labvec=NULL	
             } else {
               stop("NOTE: 'size' must be provided as a vector (numeric)!")
             }
             # label vec
             if(is.null(labvec))labvec=as.character(sizevec)	
             if(length(labvec)!=length(sizevec)){
               stop("NOTE: 'labvec' and 'sizevec' must have the same length!")
             }
             labvec=as.character(labvec)
             
             # colors
             if(!is.null(colvec) && length(colvec)>0 && is.character(colvec)){
               if(!is.color(colvec)){
                 warning("NOTE: invalid 'color'")
                 colvec='#000000'
               } else {
                 colvec <- colorRampPalette(colvec, alpha=TRUE)(length(colvec))
               }      
             } else {
               colvec='#000000'
             }
             if(length(colvec)!=length(sizevec)){
               colvec <- rep(colvec[1], length(sizevec))
             }
             
             # further args
             if(!is.null(position) && !is.character(position))position=NULL
             position=position[1]
             if(!is.null(dxborder) && !is.numeric(dxborder))dxborder=NULL
             dxborder=dxborder[1]
             if(!is.null(dyborder) && !is.numeric(dyborder))dyborder=NULL
             dyborder=dyborder[1]
             if(!is.null(vertical) && !is.logical(vertical))vertical=NULL
             vertical=vertical[1]
             if(!is.null(ftsize) && !is.numeric(ftsize))ftsize=NULL
             ftsize=ftsize[1]
             if(!is.null(title) && !is.character(title))title=NULL
             title=title[1]	
             if(!is.null(dxtitle) && !is.numeric(dxtitle))dxtitle=NULL
             dxtitle=dxtitle[1]
             if(!is.null(edgelen) && !is.numeric(edgelen))edgelen=NULL
             edgelen=edgelen[1]
             if(!is.null(intersp) && !is.numeric(intersp)) intersp=NULL
             intersp=intersp[1]
             
             if(!is.null(position)){
               position=switch(position, bottomright="bottomRight", 
                               bottomleft="bottomLeft", topright="topRight", topleft="topLeft", NULL)
             }
             
             # default settings
             if(type=="nodesize"){
               if(is.null(position))position="bottomLeft"
               if(is.null(dxborder))dxborder=5
               if(is.null(dyborder))dyborder=5
               if(is.null(vertical))vertical=FALSE
               if(is.null(ftsize))ftsize=12
               if(is.null(title))title="nodesize"
               if(is.null(dxtitle))dxtitle=5
               if(is.null(intersp))intersp=3	
               if(is.null(edgelen))edgelen=0 #not used for nodes!
             } else {
               if(is.null(position))position="topLeft"
               if(is.null(dxborder))dxborder=10
               if(is.null(dyborder))dyborder=5
               if(is.null(vertical))vertical=TRUE
               if(is.null(ftsize))ftsize=12
               if(is.null(title))title="edgewidth"
               if(is.null(dxtitle))dxtitle=5
               if(is.null(intersp))intersp=10
               if(is.null(edgelen))edgelen=50
             }
             vertical=ifelse(vertical,"true","false")
             
             invisible( .rederexpresspost(obj, 'RedHandler.addLegendSize', 
                                          sizevec, labvec, colvec, intersp, 
                                          ftsize, title, dxtitle, position, 
                                          dxborder, dyborder, vertical, type,
                                          edgelen ) )
             
           }
)

#-------------------------------------------------------------------------------
setMethod ('addLegend.shape', 'RedPort', 
           function (obj, shapevec, type="node", labvec=NULL, colvec=NULL, 
                     position=NULL, dxborder=NULL, dyborder=NULL, vertical=TRUE, 
                     ftsize=NULL, title=NULL, dxtitle=NULL, size=NULL, intersp=NULL) {
             
             if(ping(obj)==0)return(invisible())
             
             #check loaded igraph
             igraph.check()
             
             # checks---------------------------------------------------- 
             
             if(!is.character(type))type="nodeshape"
             type=switch(type, node="nodeshape", edge="edgeshape", "nodeshape")
             
             #Check if igraph object------------------------------------------
             if(igraph::is_igraph(shapevec)){
               if(type=="nodeshape"){
                 if(!is.null(G(shapevec,"legNodeShape")$shape)){
                   if(!is.null(G(shapevec,"legNodeShape")$legend) && is.null(labvec)){
                     labvec=G(shapevec,"legNodeShape")$legend
                     if(is.null(title))title=G(shapevec,"legNodeShape")$title
                   }
                   shapevec=G(shapevec,"legNodeShape")$shape
                 } else {
                   stop("NOTE: there is no valid 'legNodeShape' legend information for this igraph object!!")
                 }
               } else {
                 if(!is.null(G(shapevec,"legEdgeType")$shape)){
                   if(!is.null(G(shapevec,"legEdgeType")$legend) && is.null(labvec)){
                     labvec=G(shapevec,"legEdgeType")$legend
                     if(is.null(title))title=G(shapevec,"legEdgeType")$title
                   }
                   shapevec=G(shapevec,"legEdgeType")$shape
                 } else {
                   stop("NOTE: there is no valid 'legEdgeType' legend information for this igraph object!!")
                 }			
               }
             }
             
             #shapes
             defaultv=c('ELLIPSE', 'RECTANGLE', 'ROUNDED_RECTANGLE', 'TRIANGLE', 'DIAMOND')
             defaulte=c('LONG_DASH','SOLID','DOTTED','DASHED')
             if(type=='nodeshape'){
               defaultshapes=defaultv
             } else {
               defaultshapes= defaulte
             }
             if(!is.null(shapevec) && length(shapevec)>1){       
               c1=!is.character(shapevec)
               if(c1){
                 stop("NOTE: 'shape' must be provided as character!")
               } else if(sum(is.na(shapevec))>0){
                 stop("NOTE: invalid 'shape': 'NA' found'!")
               } else if(sum(!shapevec%in%defaultshapes)>0){
                 stop(paste("NOTE: invalid 'shape' input. Options:", paste(defaultshapes, collapse="  ")))
               } else {            
                 shapevec=as.character(shapevec)
               }
             } else if(is.null(shapevec)){
               type="null"
               shapevec=c("null","null")	
               labvec=NULL	
             } else {
               stop("NOTE: 'shape' must be provided as a vector (character)!")
             }
             # label vec
             if(is.null(labvec))labvec=letters[c(1:length(shapevec))]
             if(length(labvec)!=length(shapevec)){
               stop("NOTE: 'labvec' and 'shapevec' must have same length!")
             }
             labvec=as.character(labvec)
             
             # colors
             if(!is.null(colvec) && length(colvec)>0 && is.character(colvec)){
               if(!is.color(colvec)){
                 warning("NOTE: invalid 'color'")
                 colvec='#000000'
               } else {
                 colvec <- colorRampPalette(colvec, alpha=TRUE)(length(colvec))
               }      
             } else {
               colvec='#000000'
             }
             if(length(colvec)!=length(shapevec)){
               colvec <- rep(colvec[1], length(shapevec))
             }
               
             # further args
             if(!is.null(position) && !is.character(position))position=NULL
             position=position[1]
             if(!is.null(dxborder) && !is.numeric(dxborder))dxborder=NULL
             dxborder=dxborder[1]
             if(!is.null(dyborder) && !is.numeric(dyborder))dyborder=NULL
             dyborder=dyborder[1]
             if(!is.null(vertical) && !is.logical(vertical))vertical=NULL
             vertical=vertical[1]
             if(!is.null(ftsize) && !is.numeric(ftsize))ftsize=NULL
             ftsize=ftsize[1]
             if(!is.null(title) && !is.character(title))title=NULL
             title=title[1]	
             if(!is.null(dxtitle) && !is.numeric(dxtitle))dxtitle=NULL
             dxtitle=dxtitle[1]
             if(!is.null(size) && !is.numeric(size))size=NULL
             size=size[1]	
             if(!is.null(intersp) && !is.numeric(intersp)) intersp=NULL
             intersp=intersp[1]
             
             if(!is.null(position)){
               position=switch(position, bottomright="bottomRight", 
                               bottomleft="bottomLeft", 
                               topright="topRight", topleft="topLeft", NULL)
             }
             
             # default settings
             if(type=="nodeshape"){
               if(is.null(position))position="bottomRight"
               if(is.null(dxborder))dxborder=5
               if(is.null(dyborder))dyborder=5
               if(is.null(vertical))vertical=TRUE
               if(is.null(ftsize))ftsize=12
               if(is.null(title))title="nodeshape"
               if(is.null(dxtitle))dxtitle=5
               if(is.null(size))size=20
               if(is.null(intersp))intersp=5	
             } else {
               if(is.null(position))position="bottomRight"
               if(is.null(dxborder))dxborder=10
               if(is.null(dyborder))dyborder=150
               if(is.null(vertical))vertical=TRUE
               if(is.null(ftsize))ftsize=12
               if(is.null(title))title="edgetype"
               if(is.null(dxtitle))dxtitle=5
               if(is.null(size))size=4
               if(is.null(intersp))intersp=15
             }
             vertical=ifelse(vertical,"true","false")
             
             invisible( .rederexpresspost(obj, 'RedHandler.addLegendShape', 
                                          shapevec, labvec, colvec, size, 
                                         intersp, ftsize, title, dxtitle, position, 
                                         dxborder, dyborder, vertical, type) )
             
           }
)

################################################################################
### Internal Functions
################################################################################

#-------------------------------------------------------------------------------
#fix an attribute conflict between igraph/0 versions!
G <- function(g,att){
  igraph::graph_attr(g,att)
}
#-------------------------------------------------------------------------------
.getNodeAliases<-function (obj, status="all", type="node") { 
  return ( .rederpost(obj, 'RedHandler.getNodeAliases', type, status) )
}
#-------------------------------------------------------------------------------
.getNodeX<-function(obj, status="all", type="node"){
  return (.rederpost(obj, 'RedHandler.getNodeX', type, status))
}
#-------------------------------------------------------------------------------
.getNodeY<-function(obj, status="all", type="node"){
  return (.rederpost(obj, 'RedHandler.getNodeY', type, status))
}
#-------------------------------------------------------------------------------
.getNodeBend<-function(obj, status="all", type="node"){
  return (.rederpost(obj, 'RedHandler.getNodeBend', type, status))
}
#-------------------------------------------------------------------------------
.getNodeSize<-function(obj, status="all", type="node"){
  return (.rederpost(obj, 'RedHandler.getNodeSize', type, status))
}
#-------------------------------------------------------------------------------
.getNodeShape<-function(obj, status="all", type="node"){
  return (.rederpost(obj, 'RedHandler.getNodeShape', type, status))
}
#-------------------------------------------------------------------------------
.getNodeColor<-function(obj, status="all", type="node"){
  return (.rederpost(obj, 'RedHandler.getNodeColor', type, status))
}
#-------------------------------------------------------------------------------
.getNodeLineWidth<-function(obj, status="all", type="node"){
  return (.rederpost(obj, 'RedHandler.getNodeLineWidth', type, status))
}
#-------------------------------------------------------------------------------
.getNodeLineColor<-function(obj, status="all", type="node"){
  return (.rederpost(obj, 'RedHandler.getNodeLineColor', type, status))
}
#-------------------------------------------------------------------------------
.getNodeFontSize<-function(obj, status="all", type="node"){
  return (.rederpost(obj, 'RedHandler.getNodeFontSize', type, status))
}
#-------------------------------------------------------------------------------
.getNodeFontColor<-function(obj, status="all", type="node"){
  return (.rederpost(obj, 'RedHandler.getNodeFontColor', type, status))
}
#-------------------------------------------------------------------------------   
.getNodeWeight<-function(obj, status="all", type="node"){
  return (.rederpost(obj, 'RedHandler.getNodeWeight', type, status))
}
#-------------------------------------------------------------------------------
.getArrowDirection<-function(obj, status="all", type="node"){
  return (.rederpost(obj, 'RedHandler.getArrowDirection', type, status))
}
#-------------------------------------------------------------------------------
.getEdgeWidth<-function(obj, status="all", type="node"){
  return (.rederpost(obj, 'RedHandler.getEdgeWidth', type, status))
}
#-------------------------------------------------------------------------------
.getEdgeColor<-function(obj, status="all", type="node"){
  return (.rederpost(obj, 'RedHandler.getEdgeColor', type, status))
}
#-------------------------------------------------------------------------------
.getEdgeType<-function(obj, status="all", type="node"){
  return (.rederpost(obj, 'RedHandler.getEdgeType', type, status))
}
#-------------------------------------------------------------------------------
.getEdgeWeight<-function(obj, status="all", type="node"){
  return (.rederpost(obj, 'RedHandler.getEdgeWeight', type, status))
}
##------------------------------------------------------------------------------
is.singleNumber <- function(para){
  (is.integer(para) || is.numeric(para)) && length(para)==1L && !is.na(para)
}
is.singleInteger <- function(para){
  lg <- (is.integer(para) || is.numeric(para)) && length(para) == 1L && !is.na(para)
  if(lg) lg <- ( (para+1) / (ceiling(para)+1) ) == 1
  return(lg)
}
is.singleString <- function(para){
  is.character(para) && length(para) == 1L && !is.na(para)
}
is.singleLogical <- function(para){
  is.logical(para) && length(para) == 1L && !is.na(para)
}
all.binaryValues <- function(para){
  all( para %in% c(0, 1, NA) )
}
all.integerValues <- function(para){
  lg <- ( all(is.integer(para)) || all(is.numeric(para)) ) && 
    !any(is.na(para))
  if(lg) lg <- all(( (para+1) / (ceiling(para)+1) ) == 1)
  return(lg)
}
all.characterValues <- function(para){
  all(is.character(para)) && !any(is.na(para))
}
is.color <- function(x){
  res <- try(col2rgb(x, alpha=TRUE),silent=TRUE)
  return(!"try-error"%in%class(res))
}

#-------------------------------------------------------------------------------
#--- deprecated methods
setMethod ('addSubgraph.list', 'RedPort', 
           function (obj, g, nodeList) {
             
             if(ping(obj)==0)return(invisible())
             
             .Deprecated(new="addGraph/getGraph related methods",old="addSubgraph.list") 
             
           }
)
setMethod ('addSubgraph', 'RedPort', 
           function (obj, g, nodes) {
             
             if(ping(obj)==0)return(invisible())
             
             .Deprecated(new="addGraph/getGraph related methods",old="addSubgraph")              
             
           }
)
setMethod ('duplicateGraph', 'RedPort', 
           function (obj) {
             
             if(ping(obj)==0)return(invisible())
             
             .Deprecated(new="addGraph/getGraph related methods",old="addSubgraph")  
             
           }
)
setMethod ('addSeries', 'RedPort', 
           function (obj, g) {
             
             if(ping(obj)==0)return(invisible())
             
             .Deprecated(new="addGraph/getGraph related methods",old="addSeries")  
             
           }
)


